/***************************************************************************
    file	         : kb_libloader_qt3.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/



/*  This file contains the library loader code for Linux/QT3 builds of	*/
/*  Rekall. By preference it would be in libkbase_kde, but in order	*/
/*  that libkbase_common does not have any depend on any other Rekall	*/
/*  libraries (so that the drivers can be used outside of Rekall) we	*/
/*  have to put it here.						*/

#if		__KB_TKC

#include	<qglobal.h>

#include	"kb_classes.h"
#include	"kb_libloader_qt3.h"

#include	<dlfcn.h>
#define		DLERROR()	dlerror()
#define		LIBEXTN		".so"


#define		__DEBUG_LIBLOADER	(0)
#if		__DEBUG_LIBLOADER
#include	"kb_dprintf.h"
#endif

/*  On Qt3/Linux we use "dlopen" rather than QLibrary since in the	*/
/*  "::getGlobalLibrary" method we need to use RTLD_GLOBAL (which is	*/
/*  needed for the Python scripting interface), which does not seem to	*/
/*  be available via QLibrary.						*/


void	*KBLibrary::factory ()
{
	typedef	void	*(*IFPROC)()	;

	DPRINTF
	((	"KBLibrary::factory: resolve [%s]\n",
		(const char *)m_name
	))	;

	IFPROC	ifproc	= (IFPROC)dlsym (m_library, "init_" + m_name) ;
	if (ifproc == 0)
	{
		DPRINTF
		((	"KBLibrary::factory: no interface %s [%s]\n",
			(const char *)m_name,
			DLERROR()
		))	;
		return	0 ;
	}

	return	(*ifproc)() ;

}

/*  KBLibLoader								*/
/*  self	: Instantiate the library loader			*/
/*  (returns)	: KBLibLoader *	: Loader object				*/

KBLibLoader
	*KBLibLoader::self ()
{
	static	KBLibLoader	*loader	;
	if (loader == 0) loader = new KBLibLoader ;
	return	loader	;
}

/*  KBLibLoader								*/
/*  getLibrary	: Load a library					*/
/*  libname	: const QString & : Library name			*/
/*  (returns)	: KBLibrary *	  : Library object or null on error	*/

KBLibrary
	*KBLibLoader::getLibrary
	(	const QString	&libname
	)
{
	DPRINTF
	((	"KBLibLoader::getLibrary: [%s]\n",
		(const char *)libname
	))	;
	void	*lib	= dlopen (libname + LIBEXTN, RTLD_NOW) ;
	if (lib == 0)
	{
		m_error	= DLERROR() ;
		return	0 ;
	}

	DPRINTF
	((	"KBLibLoader::getLibrary: loaded [%s]\n",
		(const char *)libname
	))	;

	return	new KBLibrary (lib, libname) ;
}

/*  KBLibLoader								*/
/*  getGlobalLibrary							*/
/*		: Load a library with global symbol access		*/
/*  libname	: const QString & : Library name			*/
/*  (returns)	: KBLibrary *	  : Library object or null on error	*/


KBLibrary
	*KBLibLoader::getGlobalLibrary
	(	const QString	&libname
	)
{
	DPRINTF
	((	"KBLibLoader::getGlobalLibrary: [%s]\n",
		(const char *)libname
	))	;
	void	*lib	= dlopen (libname + LIBEXTN, RTLD_GLOBAL|RTLD_NOW) ;
	if (lib == 0)
	{
		m_error	= DLERROR() ;
		return	0 ;
	}

	return	new KBLibrary (lib, libname) ;
}

QString	KBLibLoader::lastErrorMessage ()
{
	return	m_error	;
}


/*  ------------------------------------------------------------------  */

KBPartFactory::KBPartFactory ()
{
}

KBPartFactory::~KBPartFactory ()
{
}

QObject	*KBPartFactory::create
	(	QObject *		,
		const char *		,
		const char *		,
		const QStringList &
	)
{
	return	0 ;
}

QString KBPartFactory::ident ()
{
	return	"Unspecified plugin identifier" ;
}


KBFactory::KBFactory ()
{
}

KBFactory::~KBFactory ()
{
}

QObject	*KBFactory::create
	(	QObject *		,
		const char *		,
		const char *		,
		const QStringList &
	)
{
	return	0 ;
}

QString	KBFactory::ident ()
{
	return	0 ;
}

#endif	// __KB_TKC
