/***************************************************************************
    file	         : kb_docroot.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef		_KB_DOCROOT_H
#define		_KB_DOCROOT_H

#include	"libkbase_exports.h"

#include 	<qptrlist.h>
#include 	<qstringlist.h>
#include 	<qguardedptr.h>

#include	"kb_classes.h"
#include	"kb_location.h"
#include	"kb_attrdict.h"
#include	"kb_qrybase.h"


class	KBMimeSourceFactory	;
class	KBProgressBox		;
class	QLabel			;
class	KBSkin			;
class	KBSkinElement		;

/*  ------------------------------------------------------------------  */

/*  KBDocRoot								*/
/*  ---------								*/
/*  This class is used at the top-most node in parsed documents, such	*/
/*  as forms and reports. It holds information, such as the document	*/
/*  location, that is common to all such documents.			*/

class	LIBKBASE_API	KBDocRoot : public QObject
{
	Q_OBJECT

	KBNode			*m_parent	; /* Embedding node ...	*/
	QPtrList<KBNode>	&m_children	; /* ... and children	*/
	bool			m_donePDlg	; /* Param. dialog run	*/
	bool			m_gotExecError	; /* Exec. error flag	*/

 	QStringList		m_scripts	; /* List of scripts	*/
 	QStringList		m_scripts2	; /* List of L2 scripts	*/
	QStringList		m_imports	; /* List of imports	*/

	KBSkin			*m_skin		; /* Current skin	*/

	bool			m_loading	; /* Data being loaded	*/
	QGuardedPtr<QLabel>	m_message	; /* Status bar message	*/
	QGuardedPtr<QLabel>	m_locking	; /* Status bar lock	*/
	QGuardedPtr<KBProgressBox>
				m_progress	; /* Status progress	*/

	KBMimeSourceFactory	*m_mimeSource	;

	QDict<KBNode>		m_namedNodeMap	;

	void			buildNodeMap	(KBNode   *)	;
	void			skinChanged	(KBObject *)	;
	void			loadSkin	()		;

	bool			loadScriptModules
				(	KBScriptIF		*scriptIF,
					const QString		&language,
					const QStringList	&scripts,
					KBError			&pError
				)	;

signals	:

	void			execError 	()		;
	void			requestClose	(int)		;
	void			layoutChanged	()		;

protected slots	:

	void			slotSkinChanged	(const KBLocation &)	;

protected :

	KBLocation		m_location	; /* Database location	*/
	KBServerInfo		*m_serverInfo	; /* Server information	*/
	KBScriptIF		*m_scrIface[2]	; /* Script interfaces	*/
	KBAttrDict		*m_paramDict	; /* Param. dictionary	*/


public	:

	KBDocRoot	(KBNode *, QPtrList<KBNode> &, const KBLocation &) ;
       ~KBDocRoot	() ;

	KBDBInfo	 	*getDBInfo	() ;
	const KBLocation 	&getLocation	() ;
	const KBServerInfo 	*getServerInfo	() ;

	void			reset		() ;

	KBScriptIF		*getScriptIF	(bool, KBError &) ;
	KBScriptIF	 	*loadScripting	(KBError &) ;
	KBScriptIF	 	*loadScripting2	(KBError &) ;

	KB::ShowRC		setParamDict	(const QDict<QString> &, KBError &) ;
	const KBAttrDict	&parameterDict	() ;
	QString			getParamValue	(cchar *) ;
	void			setParamValue	(cchar *, const QString &) ;

	const QString		&appFont	() ;
	QString			skinName	() ;
	KBServerInfo::TestsEnable
				showTests	() ;

	void			setStatusBar	(QLabel *, QLabel *, KBProgressBox *) ;
	KBMimeSourceFactory	*mimeSource	() ;

	const QStringList	&getScripts	() ;
	const QStringList	&getImports	() ;

	void			loadingStart	() ;
	void			loadingDone	() ;
	bool			loadingProgress	(uint, uint, QString = QString::null) ;

	bool			isLoading	() ;

	void			doExecError	() ;
	void			doRequestClose	(int) ;
	void			doSetMessage	(const QString    &) ;
	void			doSetLocking	(KBQryBase::Locking) ;
	void			doLayoutChanged	() ;

	void			skinChanged	() ;

	const QString		&skinFGColor	(const QString &) ;
	const QString		&skinBGColor	(const QString &) ;
	const QString		&skinFont	(const QString &) ;
	const QDict<KBSkinElement>
				&skinElements 	() ;

	KBNode			*getNamedNode	(const QString &) ;

	/* The following method indicates whether an execution error	*/
	/* has occurred. This is returned as a (const) reference to the	*/
	/* error flag for reasons of efficiency on the scripting	*/
	/* interfaces, viz:						*/
	/*								*/
	/*	const bool &e = xxx->gotExecError() ;			*/
	/*	if (e) return ....	;				*/
	/* 	make_api_call ()	;				*/
	/*	if (e) return ....	;				*/
	/*								*/
	/* Hence, error occurance can be checked before and after the	*/
	/* API call at minimum cost.					*/

	const bool	&gotExecError		() ;
}	;


#endif		// _KB_DOCROOT_H
