/***************************************************************************
    file	         : kb_intelli.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qapplication.h>
#include	<qcursor.h>
#include	<qlabel.h>
#include	<qfontmetrics.h>

#include	"rk_listbox.h"
#include	"kb_classes.h"
#include	"kb_textedit.h"
#include	"kb_ascii.h"

#if		__KB_AUTOMOC
#include	"kb_intelli.moc"
#else
#include	"kb_intelli.h"
#endif


class	KBIntelliItem : public QListBoxText
{
public	:

	KBMethDictEntry	*m_method	;

	inline	KBIntelliItem
		(	RKListBox	*parent,
			KBMethDictEntry	*method
		)
		:
		QListBoxText	(parent, method->description()),
		m_method	(method)
	{
	}
}	;

/*  KBIntelli								*/
/*  KBIntelli	: Constructor for syntax helper widget			*/
/*  editor	: KBTextEdit *	  : Editor widget			*/
/*  									*/
/*  (returns)	: QWidget *	  : Helper widget			*/

KBIntelli::KBIntelli
	(	KBTextEdit		*editor,
		QPtrList<KBMethDictEntry>	&methods
	)
	:
	RKVBox
	(	editor,
		"kbintelli",
		WType_TopLevel|WStyle_Customize|WStyle_NoBorder|WStyle_StaysOnTop
	),
	m_editor (editor)
{
	int		maxWid	= 0 ;
	QFontMetrics	fm	(font()) ;

	fprintf
	(	stderr,
		"KBIntelli::KBIntelli: showing %d\n",
		methods.count()
	)	;

	setFrameStyle	(QFrame::Box|QFrame::Plain) ;
	setLineWidth	(2) ;
	setMargin	(2) ;

	m_listBox = new RKListBox (this) ;

	connect
	(	m_listBox,
		SIGNAL	(doubleClicked(QListBoxItem *)),
		SLOT	(slotChosen   (QListBoxItem *))
	)	;

	for
	(	KBMethDictEntry *entry = methods.first() ;
		entry != 0 ;
		entry  = methods.next()
	)
	{
		int w = fm.width(entry->description()) ;
		if (w > maxWid) maxWid = w ;

		new KBIntelliItem (m_listBox, entry) ;
	}

	qApp	 ->installEventFilter (this) ;
	m_listBox->installEventFilter (this) ;

	connect
	(	&m_timer,
		SIGNAL	(timeout    ()),
		SLOT	(slotTimeout())
	)	;

	setMinimumWidth (maxWid + 16)	;
	setCaption	("Rekall")	;
	polish	   	()		;



	QPoint	absPos	= m_editor->textCursorPoint() ;
	QSize	size	= sizeHint	() ;
	int	x	= absPos.x	() ;
	int	y	= absPos.y	() + 10 ;
	int	w	= size  .width 	() ;
	int	h	= size  .height	() ;

        int     lheight = m_editor->lineHeight () ;

	QWidget	*dt	= QApplication::desktop() ;
	int	sx	= dt->x     () ;
	int	sy	= dt->y     () ;
	int	sw	= dt->width () ;
	int	sh	= dt->height() ;

        if (x + w > sw) x  = sw - w ;
        if (y + h > sh) y  = y - lheight - 2 * 6 - h ;

        if (x     < sx) x = sx ;
        if (y     < sy) y = sy ;


//	fprintf
//	(	stderr,
//		"KBIntelli::KBIntelli: sizeHint=%s at (%d,%d)\n",
//		_TEXT(sizeHint()),
//		x,
//		y
//	)	;

	move	(x, y)	;
	show	()	;

}

/*  KBIntelli								*/
/*  eventFilter	: Event filter for things we need to Know About(tm)	*/
/*  o		: QObject *	: Target object				*/
/*  e		: QEvent *	: Event in question			*/
/*  (returns)	: bool		: Event consumed			*/

bool	KBIntelli::eventFilter
	(	QObject		*o,
		QEvent		*e
	)
{
	if (!o->isWidgetType()) 
		return	QFrame::eventFilter (o, e) ;

	if (o == m_listBox)
	{
		switch (e->type())
		{
			case QEvent::KeyPress	:
			case QEvent::KeyRelease	:
//				fprintf
//				(	stderr,
//					"KBIntelli::event: rerouting key event\n"
//				)	;

				QApplication::sendEvent (m_editor->textEdit(), e) ;
				return	true	;

			default	:
				break	;
		}

		return	QFrame::eventFilter (o, e) ;
	}


	/* The kill flag will be set if we decide that we need to	*/
	/* undertake voluntary euthenasia ...				*/
	bool	kill	= false	;
	QWidget	*w	= 0	;

	switch (e->type())
	{
		case QEvent::FocusIn	:
			/* Focus-out kills us unless the focus is	*/
			/* moving into us or into the parent editor.	*/
			kill	= true ;
			for (w = (QWidget *)o ; w != 0 ; w = w->parentWidget())
				if ((w == this) || (w == m_editor))
				{	kill	= false	;
					break	;
				}
			break	;

		case QEvent::WindowDeactivate	:
			/* If deactivating either this widget or the	*/
			/* editor then we should die, unless the other	*/
			/* one is about to be activated ... so start a	*/
			/* timer.					*/
			if ((o == this->topLevelWidget()) || (o == m_editor->topLevelWidget()))
			{
//				fprintf
//				(	stderr,
//					"KBIntelli::eventFilter: start timer\n"
//				)	;
				m_timer.start (100, false) ;
			}
			break	;

		case QEvent::WindowActivate	:
			/* If activating us or the editor then cancel	*/
			/* any deactivation timer.			*/
			if ((o == this->topLevelWidget()) || (o == m_editor->topLevelWidget()))
			{
//				fprintf
//				(	stderr,
//					"KBIntelli::eventFilter: stop  timer\n"
//				)	;
				m_timer.stop  () ;
			}
			break	;

		default	:
			break	;
	}

	if (kill)
		deleteLater ()	;

	return	QFrame::eventFilter (o, e) ;
}

void	KBIntelli::slotTimeout ()
{
	deleteLater ()	;
}

void	KBIntelli::slotChosen
	(	QListBoxItem	*item
	)
{
	if (item != 0)
		emit sigChosen (((KBIntelliItem *)item)->m_method) ;
}
