/***************************************************************************
    file	         : kb_keymapper.h
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#ifndef	__KB_KEYMAPPER_H
#define	__KB_KEYMAPPER_H

#include	<qobject.h>
#include	<qstring.h>
#include	<qintdict.h>
#include	<qvaluelist.h>
#include	<qguardedptr.h>

#include	"libkbase_exports.h"


#include	"kb_intelli.h"

class		KBKeyMapperMap	;
class		KBTextEdit	;
class		KBMethDictEntry	;
class		KBNode		;

/*  KBKeyMapper								*/
/*  -----------								*/
/*  This is an abstract base class that provides a mechanism for	*/
/*  binding aribtrary key sequences to actions, such as for an editor.	*/
/*  It is inspired by the key binder for the SSED editor.		*/

class	LIBKBASE_API KBKeyMapper : public QObject
{
	Q_OBJECT

	static	QIntDict<KBKeyMapperMap> m_map		;

	KBKeyMapperMap			*m_currentMap	;
	bool				m_gobble	;

	static	QValueList<int>		keysToKeys	(const QString &) ;
	KBKeyMapperMap			*findMapperMap	(const QValueList<int> &);

protected:

	virtual int		codeToCode	(const QString &) 	= 0 ;
	virtual	bool		helper		(int, const QString &)	= 0 ;
	virtual	bool		function	(int, const QString &)	= 0 ;
	virtual	void		error		() ;

	virtual	bool		useAtOuterLevel	(int, int) ;

public:

	KBKeyMapper		(QObject *)	;
	virtual	~KBKeyMapper	()		;

	void			bindFunction	(const QValueList<int> &, int, const QString & = QString::null);
	void			bindHelper	(const QValueList<int> &, int, const QString & = QString::null);

	bool			applyKey	(int, int) ;
	void			clearKey	() ;

	void			clearKeyMap	() ;
	QString 		loadKeyMap	(const QString &);

	friend class KBKeyMapperMap ;
}	;

/*  KBTextEditMapper								*/
/*  ---------------								*/
/*  Implementation of tke KBKeyMapper for use with the KBTextEdit (aka	*/
/*  QTextEdit/KTextEdit) text editor.					*/

class	LIBKBASE_API	KBTextEditMapper : public KBKeyMapper
{
	Q_OBJECT

	enum	MatchType
	{	MatchAny,
		MatchExact
	}	;

	KBTextEdit		*m_editor	;
	KBNode			*m_eventNode	;

	QGuardedPtr<KBIntelli>	m_helper	;
	int			m_helperPara	;
	int			m_helperCol	;
	QString			m_helperText	;
	MatchType		m_helperType	;


	void			showHelper	(QPtrList<KBMethDictEntry> &, const QString &, MatchType) ;
	void			hideHelper	() ;

protected slots :

	void			checkChangeLine	() ;
	void			slotChosen	(KBMethDictEntry *) ;

protected :

	virtual	bool		useAtOuterLevel	(int, int) ;
	virtual int		codeToCode	(const QString &) ;
	virtual	bool		helper		(int, const QString &) ;
	virtual	bool		function	(int, const QString &) ;

	QString			scanForMethod	() ;

public	:

	KBTextEditMapper	(KBTextEdit *) ;
       ~KBTextEditMapper	()	;

	bool	processEvent	(QEvent *) ;

	inline	void	setEventNode
		(	KBNode		*node
		)
	{
		m_eventNode = node	;
	}

	enum	Action
	{	NoAction,
		StartOfLine,
		EndOfLine,
		StartOfDoc,
		EndOfDoc,
		NextWord,
		PreviousWord,
		DelWordLeft,
		DelWordRight,
		DelLine,
		Cut,
		Copy,
		Paste,
		ToLower,
		ToUpper,
		ScrollUp,
		ScrollDown,
		Undo,
		Redo,
		SearchForward,
		SearchBack
	}	;
}	;


#endif	// __KB_KEYMAPPER_H
