/***************************************************************************
    file	         : kb_object_full.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#if		! __KB_RUNTIME

#include	<stdio.h>

#include	<qapplication.h>
#include	<qpalette.h>

#include 	"kb_object.h"
#include	"kb_slot.h"
#include	"kb_ctrl.h"
#include	"kb_display.h"
#include	"kb_layout.h"
#include	"kb_formblock.h"
#include	"kb_reportblock.h"
#include	"kb_form.h"
#include	"kb_options.h"
#include	"kb_font.h"
#include	"kb_docroot.h"
#include	"kb_location.h"
#include	"kb_script.h"
#include	"kb_config.h"
#include	"kb_override.h"
#include	"kb_complink.h"
#include	"kb_quicktext.h"
#include	"kb_directprop.h"
#include	"kb_rowcoldialog.h"
#include	"kb_deleter.h"
#include	"kb_designpopup.h"
#include	"kb_ascii.h"


#include	"kb_popupmenu.h"
#include	"kb_formcopier.h"
#include	"kb_sizer.h"
#include	"kb_noobjdlg.h"
#include	"kb_component.h"
#include	"kb_compaccessdlg.h"

#include	"kb_nodereg.h"
#include	"kb_toolbox.h"

#include	"tk_fontdialog.h"
#include	"tk_colordialog.h"
#include	"tk_messagebox.h"


#define		__DEBUG_OBJECT	(0)
#if		__DEBUG_OBJECT
#include	"kb_dprintf.h"
#endif



KBWhatsThisPair::KBWhatsThisPair ()
{
}

KBWhatsThisPair::KBWhatsThisPair
	(	const QString	&legend,
		const QString	&info,
		bool		toNone
	)
	:
	m_legend (legend),
	m_info	 (info)
{
	if (m_info.isEmpty() && toNone)
		m_info = TR("<i>None</i>") ;
}

/*  ------------------------------------------------------------------	*/

/*  KBObject								*/
/*  createSizer	: Create sizer						*/
/*  (returns)	: void		:					*/

void	KBObject::createSizer ()
{
	/* HACK ALERT:							*/
	/* If this control is below a component link then we do not	*/
	/* give it a sizer (since the user cannot manipulate it) and	*/
	/* we lower it, so that it will definitely be below the		*/
	/* the component link's own sizer blobs.			*/
	for
	(	KBNode *parent = getParent() ;
		parent != 0 ;
		parent = parent->getParent()
	)
		if (parent->isCompLink() != 0)
		{	m_control->getWidget()->lower() ;
			return ;
		}

	m_sizer	= new KBSizer  (this, m_display, m_control) ;
}

/*  KBObject								*/
/*  setSizer	: Set sizer						*/
/*  sizer	: KBSizer *	: Sizer to set				*/
/*  (returns)	: void		:					*/

void	KBObject::setSizer
	(	KBSizer		*sizer
	)
{
	DELOBJ	(m_sizer) ;
	m_sizer	= sizer	;
}

/*  KBObject								*/
/*  addNewObject: Add a new object					*/
/*  rect	: QRect		: Position				*/
/*  (returns)	: bool		: Object (probably) added		*/

bool	KBObject::addNewObject
	(	QRect		rect
	)
{
	DPRINTF
	((	"KBObject::addNewObject: called rect=%s\n",
		_TEXT(rect)
	))	;
	extern	void	snapRect (QRect &) ;
	if (KBOptions::snappingOn() && !isDynamic()) snapRect (rect) ;

	setCtrlRect (rect) ;

	/* See if the toolbox yields a specification, in which case	*/
	/* the action to take depends on the spec ...			*/
	NodeSpec *spec	= KBToolBox::self()->currentSpec() ;
	if (spec != 0)
	{
		DPRINTF
		((	"KBObject::addNewObject: spec: %s\n",
			spec->m_msgText
		))	;
		/* A special case is the use-popup dummy spec in which	*/
		/* case the non-toolbox popup menu system is used.	*/
		if (KBToolBox::specIsUsePopup  (spec))
		{
			KBPopupMenu *pop = makeNewPopup (0, rect) ;
			pop->exec (QCursor::pos()) ;
			delete	pop  ;
			return	true ;
		}

		/* Check for the nominal specifications that indicate	*/
		/* a new table block, a new query block, ...		*/
		if (KBToolBox::specIsTableBlock(spec))
		{
			newTableBlock() ;
			return	true ;
		}
		if (KBToolBox::specIsQueryBlock(spec))
		{
			newQueryBlock() ;
			return	true ;
		}
		if (KBToolBox::specIsMenuBlock (spec))
		{
			newNullBlock () ;
			return	true ;
		}
		if (KBToolBox::specIsSQLBlock  (spec))
		{
			newSQLBlock  () ;
			return	true ;
		}
		if (KBToolBox::specIsContainer (spec))
		{
			newContainer () ;
			return	true ;
		}
		if (KBToolBox::specIsPasteComp (spec))
		{
			pasteComponent () ;
			return	true ;
		}
		if (KBToolBox::specIsLinkComp  (spec))
		{
			linkComponent () ;
			return	true ;
		}

		/* Not a special case, call the ::newNode with the	*/
		/* specification. The (long) cast is is there because	*/
		/* ::newNode is a slot which the popup menu system	*/
		/* invokes, whence the arguement is really a NodeSpec	*/
		/* structure.						*/
		newNode ((long)spec) ;
		return	true ;
	}

	/* No toolbox specification. If the toolbox is enabled then do	*/
	/* nothing, if not enabled then fall back on the popup context	*/
	/* menu system.							*/
	if (!KBOptions::getUseToolbox() || !KBToolBox::self()->showing())
	{
		KBPopupMenu *pop = makeNewPopup (0, rect) ;
		pop->exec (QCursor::pos()) ;
		delete	pop  ;
		return	true ;
	}

	return	false	;
}

/*  KBObject								*/
/*  selectOverlap: Select any overlaped children			*/
/*  rect	 : QRect	: Area to check				*/
/*  (returns)	 : void		:					*/

void	KBObject::selectOverlap
	(	QRect		rect
	)
{
	bool	multi	= false ;
	KBSizer	*sizer	;

	CITER
	(	Object,
		obj,
		if ((sizer = obj->overlaps (rect)) != 0)
		{	getLayout()->addSizer (sizer, multi) ;
			multi = true ;
		}
	)
}

/*  KBObject								*/
/*  checkOverlap : Check for any overlaped children			*/
/*  rect	 : QRect	: Area to check				*/
/*  (returns)	 : void		: True if any overlapped		*/

bool	KBObject::checkOverlap
	(	QRect		rect
	)
{
	CITER
	(	Object,
		obj,
		if (obj->overlaps (rect) != 0) return true ;
	)

	return	false	;
}

/*  KBObject								*/
/*  overlaps	: Return sizer if object overlaps sepecified area	*/
/*  rect	: QRect		: Area to check				*/
/*  (returns)	: KBSizer *	: Sizer or NULL				*/

KBSizer	*KBObject::overlaps
	(	QRect		rect
	)
{
	DPRINTF
	((	"KBObject::overlaps: %s %s [%s] %s\n",
		_TEXT(rect),
		_TEXT(geometry()),
		(cchar *)getName(),
		geometry().intersects (rect) ? "Yes" : "No"
	))	;
	return	geometry().intersects (rect) ? m_sizer : 0 ;
}

/*  KBObject								*/
/*  objectInCell: See if there is an object in a dynamic cell		*/
/*  rect	: QRect		: Cell					*/
/*  (returns)	: KBObject *	: Object or null if none		*/

KBObject
	*KBObject::objectInCell
	(	QRect		rect
	)
{
	CITER
	(	Object,
		obj,
		if (obj->overlaps (rect) != 0) return obj ;
	)

	return	0 ;
}

/*  KBObject								*/
/*  setCtrlRect	: set rectangle for new control				*/
/*  rect	: QRect		: Rectangle				*/
/*  (returns)	: void		:					*/

void	KBObject::setCtrlRect
	(	QRect		rect
	)
{
	DPRINTF
	((	"KBObject::setCtrlRect: %s(%d)\n",
		_TEXT(rect),
		rect.isValid()
	))	;
	m_newRect	= rect	;
}

/*  KBObject								*/
/*  newCtrlRect	: Get rectangle for new control				*/
/*  (returns)	: QRect		: Rectangle				*/

QRect	KBObject::newCtrlRect ()
{
	DPRINTF
	((	"KBObject::newCtrlRect: %s\n",
		_TEXT(m_newRect)
	))	;
	return	m_newRect	;
}

/*  KBObject								*/
/*  autoCtrlRect: Get rectangle for new control				*/
/*  big		: bool		: Default area is big			*/
/*  (returns)	: QRect		: Rectangle				*/

QRect	KBObject::autoCtrlRect
	(	bool		big
	)
{
	DPRINTF
	((	"KBObject::autoCtrlRect: %s valid=%d\n",
		_TEXT(m_newRect ),
		m_newRect.isValid()
	))	;
	if (m_newRect.isValid())
	{
		if (isDynamic())
			return m_newRect ;

		if ((m_newRect.width() >= 5) && (m_newRect.height() >= 5))
			return	m_newRect ;
	}

	int	defw	= big ? 300 : 100 ;
	int	defh	= big ? 200 :  20 ;
	return	getLayout()->autoCtrlRect (this, m_bState, QRect (10, 10, defw, defh)) ;
}

/*  KBObjects								*/
/*  insertObjectDynamic							*/
/*		: Insert child object into dynamic container		*/
/*  oNode	: KBNode *	: Likely an object			*/
/*  inDisplay	: KBDisplay *	: Owning display for objects		*/
/*  insertAt	: QRect		: Insertion range			*/
/*  (returns)	: void		:					*/

void	KBObject::insertObjectDynamic
	(	KBNode		*oNode,
		KBDisplay	*inDisplay,
		QRect		insertAt
	)
{
	KBNode		*nNode	;
	KBObject	*nObj	;

	/* HACK: We must set the geometry *before* replicating into the	*/
	/* container, else the container grid size is messed up.	*/
	/* BUT : We don't want to affect the source node we are passed.	*/
	/* SO  : Make a local duplicate first and work from that.	*/
	if ((oNode = oNode->replicate (0)) == 0)
		return	;

	if (oNode->isObject())
        	oNode->isObject()-> setGeometry (insertAt) ;

	/* Replicate the node into this block. If it is not also an	*/
	/* object then we don't need to move it.			*/
	nNode = oNode->replicate (this) ;

	DPRINTF
	((	"KBObject::insertObjectDynamic: nNode=%p\n",
		(void *)nNode
	))	;

	delete	oNode ;
	if (nNode == 0) return ;

	DPRINTF
	((	"KBObject::insertObjectDynamic: nObj =%p\n",
		(void *)nObj
	))	;

	if ((nObj  = nNode->isObject ()) == 0)
		return	;

	/* Build the display for the object, set its geometry and show	*/
	/* appropriately (should be design mode else we would not be	*/
	/* here. Note that we cll ::setGeometry() on the new object 	*/
	/* with a dummy rectangle to force control update for data	*/
	/* controls.							*/
	nObj->buildDisplay (inDisplay)		;
	nObj->setGeometry  (QRect  ())		;
	nObj->showAs   	   (isShowing())	;

	if (nObj->getContainer())
		nObj->getContainer ()->show()	;

	getLayout()->setChanged ()		;
}

/*  KBObjects								*/
/*  insertObjectStatic							*/
/*		: Insert child object into static container		*/
/*  oNode	: KBNode *	: Likely an object			*/
/*  inDisplay	: KBDisplay *	: Owning display for objects		*/
/*  insertAt	: QPoint	: Insertion position			*/
/*  (returns)	: void		:					*/

void	KBObject::insertObjectStatic
	(	KBNode		*oNode,
		KBDisplay	*inDisplay,
		QPoint		insertAt
	)
{
	KBNode		*nNode	;
	KBObject	*nObj	;

	/* Replicate the node into this block. If it is not also an	*/
	/* object then we don't need to move it.			*/
	if ((nNode = oNode->replicate (this)) == 0)
		return	;

	if ((nObj  = nNode->isObject ()) == 0)
		return	;

	/* Set the position and size of the object and adjust by the	*/
	/* offsets. If snapping is enabled then snap the result.	*/
	QRect	r  = nObj->geometry() ;
	r.moveBy (insertAt.x(), insertAt.y()) ;

	extern	void snapRect (QRect &) ;
	if (KBOptions::snappingOn()) snapRect (r) ;

	/* Build the display for the object, set its geometry and show	*/
	/* appropriately (should be design mode else we would not be	*/
	/* here.							*/
	nObj->buildDisplay (inDisplay)		;
	nObj->setGeometry  (r)			;
	nObj->showAs   	   (isShowing())	;

	if (nObj->getContainer())
		nObj->getContainer ()->show()	;

	getLayout()->setChanged ()		;
}

/*  KBObjects								*/
/*  insertObjectsStatic							*/
/*		: Insert child objects into static container		*/
/*  nodes	: QPtrList<KBNode> & : List of objects			*/
/*  inDisplay	: KBDisplay *	     : Owning display for objects	*/
/*  insertAt	: QPoint	     : Insertion position		*/
/*  (returns)	: void		     :					*/

void	KBObject::insertObjectsStatic
	(	QPtrList<KBNode>	&nodes,
		KBDisplay		*inDisplay,
		QPoint			insertAt
	)
{
	LITER
	(	KBNode,
		nodes,
		oNode,

		insertObjectStatic (oNode, inDisplay, insertAt) ;
	)
}

/*  KBObject								*/
/*  insertObjects: Insert copied or cut objects				*/
/*  inDisplay	 : KBDisplay *	  : Associated display			*/
/*  (returns)	 : void		:					*/

void	KBObject::insertObjects
	(	KBDisplay	*inDisplay
	)
{
	/* This is the simple case. Paste in any copied objects at	*/
	/* the same positions as they were wherever they came from.	*/
	/* Should only ever be called on static containers.		*/
	if (!isDynamic())
		insertObjectsStatic
		(	KBFormCopier::self()->getCopied (objType()),
			inDisplay,
			QPoint(0, 0)
		)	;
}

/*  KBObject								*/
/*  insertHere	: Insert copied or cut objects at specified position	*/
/*  nodes	: QPtrList<KBNode> & : List of nodes to paste		*/
/*  inDisplay	: KBDisplay *	     : Associated display		*/
/*  insertAt	: QRect		     : Position				*/
/*  (returns)	: void		     :					*/

void	KBObject::insertHere
	(	QPtrList<KBNode>	&nodes,
		KBDisplay		*inDisplay,
		QRect			insertAt
	)
{
	if (isDynamic())
	{
		/* If we are a dynamic container then only a single	*/
		/* object can be inserted, and none of the cells in	*/
		/* the insertion area may currently contain objects.	*/
		if (nodes.count() > 1)
		{
			KBError::EWarning
			(	TR("Can only paste single objects into a dynamic layout"),
				QString::null,
				__ERRLOCN
			)	;
			return	;
		}

		if (checkOverlap (insertAt))
		{
			KBError::EWarning
			(	TR("Each dynamic cell can only contain one object"),
				QString::null,
				__ERRLOCN
			)	;
			return	;
		}

		DPRINTF
		((	"KBObject::insertHere: dynamic: %s\n",
			_TEXT(insertAt)
		))	;

		insertObjectDynamic (nodes.at(0), inDisplay, insertAt) ;
		return	;
	}

	/* When inserting into a static container, adjust positions so	*/
	/* that the top-level object goes to the specified position.	*/
	int	minx	;
	int	miny	;
	minPosition	(nodes, minx, miny) ;
	QPoint	at	(insertAt.x() - minx, insertAt.y() - miny) ;

	DPRINTF
	((	"KBObject::insertHere: static: min=(%d,%d): %s-> %s \n",
		minx,
		miny,
		_TEXT(insertAt),
		_TEXT(at)
	))	;
	insertObjectsStatic (nodes, inDisplay, at) ;
}

/*  KBObject								*/
/*  insertComponent							*/
/*		: Insert component object at specified position		*/
/*  inDisplay	: KBDisplay *	: Associated display			*/
/*  newRect	: QRect		: New object size			*/
/*  paste	: bool		: Paste component, else link		*/
/*  (returns)	: void		:					*/

void	KBObject::insertComponent
	(	KBDisplay	*inDisplay,
		QRect		newRect,
		bool		paste
	)
{
	/* Start by running the document chooser so that the user can	*/
	/* select a set of components.					*/
	const KBLocation  &location = getRoot()->getDocRoot()->getLocation() ;

	DPRINTF
	((	"KBObject::insertComponent: f=[%p] r=[%p] %s\n",
		(void *)getRoot()->isForm  (),
		(void *)getRoot()->isReport(),
		_TEXT(newRect)
	))	;
	KBComponentLoadDlg clDlg
			   (	location.dbInfo(),
			 	location.server(),
			   	getRoot()->getAttrVal ("language"),
				newRect.size(),
				paste,
				objType()
			   )	;

	if (!clDlg.exec()) return ;


	/* First case is pasting. Retrieve the component from the 	*/
	/* dialog, then insert all objects inside the component other	*/
	/* than the null query.	The component here can the be dropped.	*/
	if (paste)
	{
		KBError	    error ;
		KBComponent *component = clDlg.component (error) ;
		if (component == 0)
		{	error.DISPLAY () ;
			return	;
		}

		const QPtrList<KBNode> &cList = component->getChildren() ;
		QPtrList<KBNode>	     iList ;

		LITER
		(	KBNode,
			cList,
			child,
			if (child->isQryNull() == 0) iList.append (child) ;
		)

		/* If this is a dynamic layout then we can only paste	*/
		/* a single object, so count the number (skipping the	*/
		/* null query) and barf if more than one is found.	*/
		if (isDynamic())
		{
			if (iList.count() > 1)
			{
				TKMessageBox::sorry
				(	0,
					QString(TR("Can only paste or link a single object into a dynamic layout")),
					TR("Pasting/Linking components")
				)	;
				return	;
			}

			iList.at(0)->isObject()->setGeometry
				(	QRect(QPoint(0,0),newRect.size())
				)	;
		}

		insertHere (iList, inDisplay, newRect) ;
		delete	component ;
		return	;
	}

	/* Second case is linking. Start by createing a new component	*/
	/* link object, which is initialised with the appropriate	*/
	/* geometry, plus the selected server and document.		*/
	KBAttrDict	aDict	;
	aDict.addValue	("x",	      newRect.x	    ()) ;
	aDict.addValue	("y",	      newRect.y	    ()) ;
	aDict.addValue	("w",	      newRect.width ()) ;
	aDict.addValue	("h",	      newRect.height()) ;
	aDict.addValue	("server",    clDlg.server  ()) ;
	aDict.addValue	("component", clDlg.document()) ;	

	bool		ok	;	
	KBCompLink	*clink	= new KBCompLink (this, aDict, &ok) ;
	if (!ok) return	;

	/* If control returns OK from the constructor, then the linked	*/
	/* component will have been loaded. At this point, gather up	*/
	/* the configuration objects and store copies immediately under	*/
	/* the component link. These will be used to update the linked	*/
	/* objects at run time.						*/
	QPtrList<KBConfig>	configSet ;
	clDlg.getAllConfigs (clink, configSet, false, true) ;

	LITER
	(	KBConfig,
		configSet,
		c,

		new KBOverride
			(	clink,
				c->ident  (),
				c->path   (),
				c->attrib (),
				c->value  (),
				c->changed()
			)	;
	)

	/* We now go on to build the display so that the user can see	*/
	/* the linked objects.						*/
	clink->buildDisplay   (inDisplay) ;
	clink->showAs         (KB::ShowAsDesign) ;

	if (clink->getContainer() != 0)
		clink->getContainer()->show () ;

	getLayout()->setChanged () ;
}

/*  KBObject								*/
/*  newContainer: Make a new child container				*/
/*  display	: KBDisplay *	: Owning display			*/
/*  (returns)	: void		:					*/

void	KBObject::newContainer
	(	KBDisplay		*display
	)
{
	QRect	   r	 = newCtrlRect () ;
	KBAttrDict cDict ;

	cDict.addValue	("x",	r.x     ()) ;
	cDict.addValue	("y",	r.y	()) ;
	cDict.addValue	("w",	r.width ()) ;
	cDict.addValue	("h",	r.height()) ;

	bool	 ok ;
	KBFramer *c = new KBContainer (this, cDict, "KBContainer", &ok) ;
	if (!ok) return ;

	c->buildDisplay	(display) ;
	c->setGeometry	(c->geometry()) ;
	c->showAs	(KB::ShowAsDesign) ;
	c->getContainer	()->show() ;

	getLayout ()->setChanged() ;
}

/*  KBObject								*/
/*  newFormBlock: Make a new child block in a form			*/
/*  display	: KBDisplay *	: Owning display			*/
/*  blkType	: int		: Block type				*/
/*  (returns)	: void		:					*/

void	KBObject::newFormBlock
	(	KBDisplay	*display,
		int		type
	)
{
	extern	KBBlock	*makeSubFormFromWizard
			(	KBObject	*,
				KBQryBase	*,
				KBBlock::BlkType,
				KBAttrDict	&,
				bool		&
			)	;

	QRect		cRect	= newCtrlRect() ;
	KBQryBase	*query	= 0 ;
	KBBlock		*block	= 0 ;

	if (KBToolBox::useWizard ((Qt::ButtonState)0))
	{
		bool		cancel	;
		KBAttrDict	aDict	;

		if (isBlock() != 0)
			query	= isBlock ()->getQuery() ;
		else	query	= getBlock()->getQuery() ;

		aDict.addValue	(cRect)	;

		block	= makeSubFormFromWizard
				(	this,
					query,
					(KBBlock::BlkType)type,
					aDict,
					cancel
				)	;

		if ((block == 0) && cancel) return ;
	}

	if (block == 0)
	{
		bool	ok  ;
		block	= new KBFormBlock
				(	this,
					cRect,
					(KBBlock::BlkType)type,
					ok
				)	;
		if (!ok)
		{
			delete	block	;
			return	;
		}
	}

	block->buildDisplay	(display ) ;
	block->setGeometry	(block->geometry()) ;
	block->showAs		(KB::ShowAsDesign ) ;
	block->getContainer	()->show() ;

	getLayout ()->setChanged() ;
}

/*  KBObject								*/
/*  newReportBlock							*/
/*		: Make a new child block in a report			*/
/*  display	: KBDisplay *	: Owning display			*/
/*  blkType	: int		: Block type				*/
/*  (returns)	: void		:					*/

void	KBObject::newReportBlock
	(	KBDisplay	*display,
		int		type
	)
{
	bool	ok  ;
	KBBlock	*b = new KBReportBlock
			 (	this,
				newCtrlRect(),
				(KBBlock::BlkType)type,
				ok
			 )	;

	if (!ok)
	{
		delete	b ;
		return	;
	}

	b->buildDisplay	(display) ;
	b->setGeometry	(b->geometry()) ;
	b->showAs	(KB::ShowAsDesign) ;
	b->getContainer	()->show() ;

	getLayout ()->setChanged() ;
}

#endif	// ! __KB_RUNTIME
