/***************************************************************************
    file	         : kb_prompt.cpp
    copyright            : (C) 1999,2000,2001,2002,2003 by Mike Richardson
			   (C) 2000,2001,2002,2003 by theKompany.com
			   (C) 2001,2002,2003 by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/



#include	<qlabel.h>
#include	<qstringlist.h>

#include	"rk_vbox.h"
#include	"rk_hbox.h"
#include	"rk_combobox.h"
#include	"rk_lineedit.h"
#include	"rk_pushbutton.h"
#include	"tk_messagebox.h"

#include	"kb_classes.h"
#include	"kb_dbinfo.h"
#include	"kb_location.h"
#include	"kb_callback.h"
#include	"kb_appptr.h"

#if		__KB_AUTOMOC
#include	"kb_prompt.moc"
#else
#include	"kb_prompt.h"
#endif


/*  KBMessageBoxYNAC							*/
/*  KBMessageBoxYNAC							*/
/*		: Constuctor for Yes/No/All/Cancel dialog		*/
/*  parent	: QWidget *	   : Parent widget			*/
/*  caption	: const QString &  : Caption text			*/
/*  buttonYes	: const QString &  : Text for yes button		*/
/*  buttonNo	: const QString &  : Text for o button			*/
/*  buttonYesAll: const QString &  : Text for yes/all button		*/
/*  buttonCancel: const QString &  : Text for cancel button		*/
/*  (returns)	: KBMessageBoxYNAC :					*/

KBMessageBoxYNAC::KBMessageBoxYNAC
	(	QWidget		*parent,
		const QString	&text,
		const QString	&caption,
		const QString	&buttonYes,
		const QString	&buttonNo,
		const QString	&buttonYesAll,
		const QString	&buttonCancel
	)
	:
	KBDialog (caption, true)
{
	RKVBox	*layMain	= new RKVBox (this) ;
	layMain->setTracking ()	;

	new QLabel (text, layMain) ;

	RKHBox	*layButt	= new RKHBox (layMain) ;
	layButt->addFiller ()	;
	RKPushButton *b0 = addButton(layButt, buttonYes,    TR("Yes"   ), SLOT(slotYes   ())) ;
	RKPushButton *b1 = addButton(layButt, buttonNo,	    TR("No"    ), SLOT(slotNo    ())) ;
	RKPushButton *b2 = addButton(layButt, buttonYesAll, TR("All"   ), SLOT(slotYesAll())) ;
	RKPushButton *b3 = addButton(layButt, buttonCancel, TR("Cancel"), SLOT(slotCancel())) ;
	layButt->addFiller ()	;

	sameSize (b0, b1, b2, b3, (void *)0) ;
}


RKPushButton
	*KBMessageBoxYNAC::addButton
	(	RKHBox		*layout,
		const QString	&text,
		const QString	&deflt,
		cchar		*slot
	)
{
	RKPushButton *b = new RKPushButton (layout) ;

	if (text.isEmpty())
		b->setText (deflt) ;
	else	b->setText (text ) ;

	connect	(b, SIGNAL(clicked()), slot) ;

	return	b ;
}

void	KBMessageBoxYNAC::slotCancel ()
{
	done	(ActCancel) ;
}

void	KBMessageBoxYNAC::slotYes ()
{
	done	(ActYes)	;
}

void	KBMessageBoxYNAC::slotNo ()
{
	done	(ActNo)	;
}

void	KBMessageBoxYNAC::slotYesAll ()
{
	done	(ActYesAll) ;
}

		
KBMessageBoxYNAC::ActRC
	KBMessageBoxYNAC::query
	(	QWidget		*parent,
		const QString	&text,
		const QString	&caption,
		const QString	&buttonYes,
		const QString	&buttonNo,
		const QString	&buttonYesAll,
		const QString	&buttonCancel
	)
{
	KBMessageBoxYNAC qb
			 (	parent,
				text,
				caption,
				buttonYes,
				buttonNo,
				buttonYesAll,
				buttonCancel
			 )	;

	return	(KBMessageBoxYNAC::ActRC)qb.exec() ;
}

KBMessageBoxYNAC::ActRC
	KBMessageBoxYNAC::query
	(	QWidget			*parent,
		const QString		&text,
		KBMessageBoxYNAC::ActRC	actRC,
		const QString		&caption,
		const QString		&buttonYes,
		const QString		&buttonNo,
		const QString		&buttonYesAll,
		const QString		&buttonCancel
	)
{
	switch (actRC)
	{
		case KBMessageBoxYNAC::ActYes	:
		case KBMessageBoxYNAC::ActNo	:
			actRC = KBMessageBoxYNAC::query
			   	(	parent,
					text,
					caption,
					buttonYes,
					buttonNo,
					buttonYesAll,
					buttonCancel
			)	;
			if (actRC == KBMessageBoxYNAC::ActCancel) return actRC ;
			if (actRC == KBMessageBoxYNAC::ActNo    ) return actRC ;
			break	;

		case KBMessageBoxYNAC::ActSingle:
			if (TKMessageBox::questionYesNo
			   	(	0,
					text,
					caption
				)
				!= TKMessageBox::Yes)
				return	KBMessageBoxYNAC::ActCancel ;
			break	;

		default	:
			break	;
	}

	return	actRC	;
}


/*  ------------------------------------------------------------------  */

/*  KBPromptDlg								*/
/*  KBPromptDlg	: Constructor for prompt dialog				*/
/*  caption	: const QString & : Dialog caption			*/
/*  message	: const QString & : Dialog prompt message		*/
/*  value	: QString & 	  : Initial value/result		*/
/*  (returns)	: KBPromptDlg	  :					*/

KBPromptDlg::KBPromptDlg
	(	const QString	&caption,
		const QString	&message,
		QString		&value
	)
	:
	KBDialog	(caption, true),
	m_value		(value)
{
	RKVBox	*layout	= new RKVBox (this) ;
	layout->setTracking () ;

	new QLabel  (message, layout) ;

	m_tValue = new RKLineEdit (layout) ;
	m_tValue->setText (value) ;

	addOKCancel (layout) ;

	m_tValue->setFocus	() ;
	m_tValue->setSelection	(0, value.length()) ;

	setMinimumSize (285, 110) ;
}

/*  KBPromptDlg								*/
/*  ~KBPromptDlg: Destructor for prompt dialog				*/
/*  (returns)	: void		:					*/

KBPromptDlg::~KBPromptDlg ()
{
}

/*  KBPromptDlg								*/
/*  accept	: Handle OK button click				*/
/*  (returns)	: void		:					*/

void	KBPromptDlg::accept ()
{
	m_value	= m_tValue->text () ;
	done (true) ;
}

/*  doPrompt	: Execute prompt dialog					*/
/*  caption	: const QString&: Dialog caption			*/
/*  message	: const QString&: Dialog prompt message			*/
/*  value	: QString &	: Initial value/result			*/
/*  (returns)	: bool		: Success				*/

bool	LIBKBASE_API	doPrompt
	(	const	QString	&caption,
		const	QString	&message,
		QString		&value
	)
{
	KBPromptDlg pDlg  (caption, message, value) ;
	return	pDlg.exec () ;
}


/*  ------------------------------------------------------------------  */

/*  KBPromptSaveDlg							*/
/*  KBPromptSaveDlg							*/
/*		: Constructor for save prompt dialog			*/
/*  caption	: const QString & : Dialog caption			*/
/*  message	: const QString & : Dialog prompt message		*/
/*  value	: QString &	  : Initial value/result		*/
/*  where	: QString &	  : Initial location/result		*/
/*  dbInfo	: KBDBInfo *	  : Database information		*/
/*  incfiles	: bool		  : Include files in list		*/
/*  (returns)	: KBPromptDlg	  :					*/

KBPromptSaveDlg::KBPromptSaveDlg
	(	const QString	&caption,
		const QString	&message,
		QString		&value,
		QString		&where,
		KBDBInfo	*dbInfo,
		bool		incfiles
	)
	:
	KBDialog	(caption, true),
	m_value		(value),
	m_where		(where),
	m_incfiles	(incfiles)
{
	m_layout = new RKVBox (this) ;
	m_layout->setTracking () ;

	new QLabel (message, m_layout) ;
	m_tValue = new RKLineEdit (m_layout) ;
	m_cWhere = new RKComboBox (m_layout) ;

	m_tValue  ->setText      (value  ) ;
	m_tValue  ->setSelection (0, value.length()) ;

	if (incfiles)
		m_cWhere->insertItem (dbInfo->getDBName()) ;

	QPtrListIterator<KBServerInfo> *svIter = dbInfo->getServerIter() ;
	KBServerInfo		    *svInfo ;
	int			    at	    = -1 ;

	while ((svInfo = svIter->current()) != 0)
	{	if (svInfo->serverName() == where) at = m_cWhere->count() ;
		m_cWhere->insertItem (svInfo->serverName()) ;
		(*svIter) += 1 ;
	}
	if (at >= 0) m_cWhere->setCurrentItem (at) ;

	setMinimumSize	  (285, 110) ;
}

/*  KBPromptSaveDlg							*/
/*  ~KBPromptSaveDlg: Destructor for prompt dialog			*/
/*  (returns)	    : void		:				*/

KBPromptSaveDlg::~KBPromptSaveDlg ()
{
}

/*  KBPromptSaveDlg							*/
/*  exec	: Execute dialog					*/
/*  (returns)	: int		: Dialog completion code		*/

int	KBPromptSaveDlg::exec ()
{
	addOKCancel (m_layout) ;
	m_tValue->setFocus  () ;

	return	KBDialog::exec () ;
}

/*  KBPromptSaveDlg							*/
/*  accept	: Handle OK button click				*/
/*  (returns)	: void		:					*/

void	KBPromptSaveDlg::accept ()
{
	m_value	= m_tValue->text () ;

	if (m_incfiles && (m_cWhere->currentItem () == 0))
		m_where	= QString (KBLocation::m_pFile) ;
	else	m_where	= m_cWhere->currentText () ;

	done (true) ;
}

/*  doPromptSave: Execute prompt save dialog				*/
/*  caption	: const QString & : Dialog caption			*/
/*  message	: const QString & : Dialog prompt message		*/
/*  value	: QString &	  : Initial value/result		*/
/*  where	: QString &	  : Initial location/result		*/
/*  dbInfo	: KBDBInfo *	  : Database information		*/
/*  incfiles	: bool		  : Include files in list		*/
/*  (returns)	: bool		  : Success				*/

bool	LIBKBASE_API	doPromptSave
	(	const QString	&caption,
		const QString	&message,
		QString		&value,
		QString		&where,
		KBDBInfo	*dbInfo,
		bool		incfiles
	)
{
	KBPromptSaveDlg pDlg  (caption, message, value, where, dbInfo, incfiles) ;
	return	pDlg.exec () ;
}



/*  ------------------------------------------------------------------  */

/*  KBChoiceDlg								*/
/*  KBChoiceDlg	: Constructor for prompt dialog				*/
/*  caption	: const QString & : Dialog caption			*/
/*  message	: const QString & : Dialog prompt message		*/
/*  choices	: QStringList   & : Choices available			*/
/*  value	: QString &	  : Initial value/result		*/
/*  (returns)	: KBChoiceDlg	  :					*/

KBChoiceDlg::KBChoiceDlg
	(	const QString	&caption,
		const QString	&message,
		QStringList	&choices,
		QString		&value
	)
	:
	KBDialog	(caption, true),
	m_value		(value)
{
	RKVBox	*layout	= new RKVBox (this) ;
	layout->setTracking () ;

	new QLabel (message, layout) ;
	m_cValues = new RKComboBox (layout) ;
	m_cValues->insertStringList (choices) ;

	addOKCancel (layout) ;

	m_cValues->setFocus () ;

	setMinimumSize (285, 110) ;
}

/*  KBChoiceDlg								*/
/*  ~KBChoiceDlg: Destructor for prompt dialog				*/
/*  (returns)	: void		:					*/

KBChoiceDlg::~KBChoiceDlg ()
{
}

/*  KBChoiceDlg								*/
/*  accept	: Handle OK button click				*/
/*  (returns)	: void		:					*/

void	KBChoiceDlg::accept ()
{
	m_value	= m_cValues->currentText () ;
	done (true) ;
}

