/***************************************************************************
    file	         : kb_qtrichtext.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#if		__KB_AUTOMOC
#include 	"kb_qtrichtext.moc"
#else
#include 	"kb_qtrichtext.h"
#endif

#include	"kb_richtext.h"
#include	"kb_block.h"
#include	"kb_display.h"
#include	"kb_docroot.h"
#include	"kb_writer.h"
#include	"kb_mimefactory.h"


#include	<qtextbrowser.h>
#include	<qcursor.h>
#include	<qstyle.h>


#define		__DEBUG_QTRICHTEXT	(0)
#if		__DEBUG_QTRICHTEXT
#include	"kb_dprintf.h"
#endif


class	KBCtrlRichTextWrapper : public QTextBrowser
{
	KBCtrlRichText	*m_richText	;

public	:

	inline	KBCtrlRichTextWrapper
		(	QWidget		*parent,
			KBCtrlRichText	*richText
		)
		:
		QTextBrowser	(parent),
		m_richText	(richText)
	{
	}

	virtual	void	emitLinkClicked
		(	const QString	&link
		)
	{
		m_richText->emitLinkClicked (link) ;
	}
}	;


/*  KBCtrlRichText							*/
/*  KBCtrlRichText							*/
/*		: Constructor for line edit wrapper			*/
/*  display	: KBDisplay *	: Parent display			*/
/*  field	: KBField *	: Parent field				*/
/*  drow	: uint		: Row number in field			*/
/*  (returns)	: KBCtrlRichText:					*/

KBCtrlRichText::KBCtrlRichText
	(	KBDisplay	*display,
		KBRichText	*richText,
		uint		drow
	)
	:
	KBControl
	(	display,
		richText,
		drow
	),
	m_richText	(richText)
{
	m_browser = new KBCtrlRichTextWrapper (display->getDisplayWidget(), this) ;

	m_browser->setMimeSourceFactory (m_richText->getDocRoot()->mimeSource()) ;
	m_browser->setTextFormat	(Qt::RichText) ;

	setupWidget (m_browser) ;
}

/*  KBCtrlRichText							*/
/*  ~KBCtrlRichText							*/
/*		: Destructor for field class				*/
/*  (returns)	:		:					*/

KBCtrlRichText::~KBCtrlRichText ()
{
}

/*  KBCtrlRichText							*/
/*  setupProperties							*/
/*		: Set properties after design change			*/
/*  (returns)	: bool		:					*/

void	KBCtrlRichText::setupProperties ()
{
	ctrlSetFrame
	(	m_browser,
		QFrame::StyledPanel|QFrame::Sunken,
		m_browser->style().pixelMetric(QStyle::PM_DefaultFrameWidth, m_browser)	
	)	;
	m_browser->setText   (QString::null) ;
	m_browser->setCursor (QCursor(Qt::ibeamCursor)) ;

#if	! __KB_RUNTIME
	if (m_showing == KB::ShowAsDesign)
	{
		if (m_drow == 0) m_browser->setText (m_richText->getName()) ;
	}
#endif
}

/*  KBCtrlRichText							*/
/*  showAs	: Set or clear design mode				*/
/*  mode	: KB::ShowAs	: Design mode				*/
/*  (returns)	: void		:					*/

void	KBCtrlRichText::showAs
	(	KB::ShowAs	mode
	)
{
	KBControl::showAs (mode) ;
	setupProperties	  ()	 ;
}

/*  KBCtrlRichText							*/
/*  setValue	: Set field value					*/
/*  value	: const KBValue &: Value				*/
/*  (returns)	: void		 :					*/

void	KBCtrlRichText::setValue
	(	const KBValue	&value
	)
{
	DPRINTF
	((	"KBCtrlRichText::setValue: %d: [%s]\n",
		m_drow,
		value.getRawText().latin1()
	))	;
	m_browser->setText  (value.getRawText()) ;
	KBControl::setValue (value) ;
}

/*  KBCtrlRichText							*/
/*  getValue	: Get field value					*/
/*  (returns)	: KBValue	: Value					*/

KBValue	KBCtrlRichText::getValue ()
{
	/* If the control is empty and the initial value was null (as	*/
	/* opposed to an empty string), then return a null string.	*/
	if (m_browser->text().isEmpty() && getIniValue().isNull())
		return	KBValue (m_richText->getFieldType()) ;

	return	KBValue (m_browser->text(), m_richText->getFieldType()) ;
}

/*  KBCtrlRichText							*/
/*  clearValue	: Clear value for specified query and display row	*/
/*  query	: bool		: Clearing for query			*/
/*  (returns)	: void		:					*/

void	KBCtrlRichText::clearValue
	(	bool	query
	)
{
	m_browser->setText    (QString::null) ;
	KBControl::clearValue (query) ;
}

/*  KBCtrlRichText							*/
/*  write	: Write field 						*/
/*  writer	: KBWriter *	: Output writer				*/
/*  rect	: QRect		: Field area				*/
/*  value	: KBValue &	: Value for reports			*/
/*  fSubs	: bool		: Substitution flag			*/
/*  extra	: int &		: Return extra space			*/
/*  (returns)	: bool		: Success				*/

bool	KBCtrlRichText::write
	(	KBWriter	*writer,
		QRect		rect,
		const KBValue	&value,
		bool		fSubs,
		int		&extra
	)
{
	if (writer->asReport())
	{
		KBWriterText *w = new KBWriterText
				  (	writer,
					rect,
					m_richText->getPalette(true),
					m_richText->getFont   (true),
					value.getRawText    (),
					0x1001,
					fSubs
				  )	;
		extra	= w->extra()	;
		return	true ;
	}

	return	KBControl::write (writer, rect, value, fSubs, extra) ;
}

/*  KBCtrlRichText							*/
/*  emitLinkClicked							*/
/*		: Link click method					*/
/*  link	: const QString & : Link				*/
/*  (returns)	: void		  :					*/

void	KBCtrlRichText::emitLinkClicked
	(	const QString	&link
	)
{
	/* Link clicking is handled locally via an event function, so	*/
	/* this is a nasty hack. In order that we can display text that	*/
	/* contains embedded URLs and *not* have QTextBrowser follow	*/
	/* them automatically, we override this method (even though it	*/
	/* is declared as private!).					*/
	DPRINTF
	((	"KBCtrlRichText::emitLinkClicked: [%d][%s] focus=[%d]\n",
		m_drow,
		link.latin1(),
		m_browser->hasFocus()
	))	;

	m_richText->linkClicked (m_richText->getBlock()->getCurDRow() + m_drow, link) ;
}

