/***************************************************************************
    file	         : kb_testsuite.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	"kb_classes.h"
#include	"kb_attr.h"

#include	"kb_node.h"
#include	"kb_docroot.h"
#include	"kb_test.h"
#include	"kb_testsuite.h"
#include	"kb_nodereg.h"
#include	"kb_testsuiteresultsdlg.h"

#include	"tk_messagebox.h"

/*  KBTestSuite								*/
/*  KBTestSuite	: Constructor for test suite node			*/
/*  parent	: KBNode *	: Parent node				*/
/*  aList	: const QDict<QString> &				*/
/*				: Attribute list			*/
/*		: bool *	:					*/
/*  (returns)	: KBTestSuite	:					*/

KBTestSuite::KBTestSuite
	(	KBNode			*parent,
		const QDict<QString>	&aList,
		bool			*
	)
	:
	KBNode		(parent, "KBTestSuite",	aList),
	m_transaction	(this,	 "transaction",	aList),
	m_maxErrors	(this,	 "maxErrors",	aList),
	m_testList	(this,	 "testList",	aList)
{
}

/*  KBTestSuite								*/
/*  KBTestSuite	: Constructor for test suite node			*/
/*  parent	: KBNode *	  : Parent node				*/
/*  extant	: KBTestSuite *	  : Extant test suite			*/
/*  (returns)	: KBTestSuite	  :					*/

KBTestSuite::KBTestSuite
	(	KBNode		*parent,
		KBTestSuite		*extant
	)
	:
	KBNode		(parent, "KBTestSuite"),
	m_transaction	(this,	 "transaction",	extant),
	m_maxErrors	(this,	 "maxErrors",	extant),
	m_testList	(this,	 "testList",	extant)
{
}

/*  KBTestSuite								*/
/*  KBTestSuite	: Destructor for test suite node			*/
/*  (returns)	:		:					*/

KBTestSuite::~KBTestSuite ()
{
}

/*  KBTestSuite								*/
/*  replicate	: Replicate this test suite				*/
/*  _parent	: KBNode *	: Parent of replicant			*/
/*  (returns)	: KBNode *	: New choice node			*/

KBNode	*KBTestSuite::replicate
	(	KBNode	*_parent
	)
{
	return	new KBTestSuite (_parent, this) ;
}

#if	! __KB_RUNTIME

bool	KBTestSuite::transaction()
{
	return	m_transaction.getBoolValue () ;
}

uint	KBTestSuite::maxErrors	()
{
	return	m_maxErrors.getIntValue () ;
}

QString	KBTestSuite::testList ()
{
	return	m_testList.getValue () ;
}

void	KBTestSuite::setTransaction
	(	bool		tranaction
	)
{
	m_transaction.setValue (tranaction) ;
}

void	KBTestSuite::setMaxErrors
	(	uint		maxErrors
	)
{
	m_maxErrors.setValue (maxErrors) ;
}

void	KBTestSuite::setTestList
	(	const QString	&listList
	)
{
	m_testList.setValue (listList) ;
}

void	KBTestSuite::execute ()
{
	KBError	    error ;
	QStringList tests = QStringList::split (",", m_testList.getValue()) ;
	KBScriptIF  *l1	  = getParent()->isDocRoot()->loadScripting (error) ;
	KBScriptIF  *l2	  = getParent()->isDocRoot()->loadScripting2(error) ;

	if (l1 != 0) l1->setTestMode (KBScriptIF::testSuite) ;
	if (l2 != 0) l2->setTestMode (KBScriptIF::testSuite) ;

	for (uint idx = 0 ; idx < tests.count() ; idx += 1)
	{
		QStringList  bits  = QStringList::split(".", tests[idx]) ;
		QString	     cname = bits[0] ;
		int	     sep   = cname.find ('/') ;

		if (sep >= 0) cname = cname.mid (sep + 1) ;

		KBNode	     *node = getParent()->getNamedNode (cname) ;

		if (node == 0)
		{
			TKMessageBox::sorry
			(	0,
				QString	(TR("Test suite %1: node '%2' not found"))
					.arg(getName())
					.arg(bits[0]),
				TR("Test suite: missing test")
			)	;
			break	;
		}

		KBTest	     *test = 0 ;
		LITER
		(	KBTest,
			node->getTests(),
			t,

			if (t->getName() == bits[1])
			{	test	= t ;
				break	;
			}
		)
		if (test == 0)
		{
			TKMessageBox::sorry
			(	0,
				QString	(TR("Test suite %1: test '%2' not found"))
					.arg(getName())
					.arg(tests[idx]),
				TR("Test suite: missing test")
			)	;
			break	;
		}

		KBValue		resval	;
		KBScriptError	*rc	;
		rc = test->execute (resval, 0, 0, false) ;
//		if (rc != 0)
//		{
//			KBScriptError::processError (rc, KBScriptError::Normal) ;
//			return	false	;
//		}
	}

	KBTestSuiteResultsDlg tsrDlg ;
	if (l1 != 0) tsrDlg.addResults (l1->getTestResults()) ;
	if (l2 != 0) tsrDlg.addResults (l2->getTestResults()) ;
	tsrDlg.exec () ;

	if (l1 != 0) l1->setTestMode (KBScriptIF::testOff) ;
	if (l2 != 0) l2->setTestMode (KBScriptIF::testOff) ;
}

#endif

NEWNODE(TestSuite, (cchar *)0, 0, KNF_FORM)
