/***************************************************************************
    file	         : kb_testsuiteresultsdlg.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	"kb_sidepanel.h"

#include	"rk_vbox.h"
#include	"rk_listview.h"
#include	"rk_pushbutton.h"

#if		__KB_AUTOMOC
#include	"kb_testsuiteresultsdlg.moc"
#else
#include	"kb_testsuiteresultsdlg.h"
#endif

static	QString	mapScriptTestResult
	(	KBScriptTestResult::TestResult	result
	)
{
	switch (result)
	{
		case KBScriptTestResult::testOK	    :
			return	TR("OK") ;

		case KBScriptTestResult::testFailed :
			return	TR("Failed") ;

		default	:
			break	;
	}

	return	QString(TR("Unknown: %1").arg((int)result)) ;
}

class	KBTestSuiteResultItem : public QListViewItem
{
	bool			m_ok		;

public	:

	virtual	void		paintCell	(QPainter *, const QColorGroup &, int, int, int) ;

	KBTestSuiteResultItem
		(	RKListView 			*,
			QListViewItem			*,
			const QString			&,
			const QString			&,
			uint				,
			const QString			&,
			KBScriptTestResult::TestResult	,
			const QString			&
		)	;
}	;

KBTestSuiteResultItem::KBTestSuiteResultItem	
	(	RKListView 			*parent,
		QListViewItem			*after,
		const QString			&server,
		const QString			&location,
		uint				lineno,
		const QString			&language,
		KBScriptTestResult::TestResult	result,
		const QString			&message
	)
	:
	QListViewItem
	(	parent,
		after,
		server,
		location,
		QString::number(lineno),
		language,
		mapScriptTestResult(result),
		message
	)
{
	m_ok = result == KBScriptTestResult::testOK ;
}

void	KBTestSuiteResultItem::paintCell
	(	QPainter	  *p,
		const QColorGroup &cg1,
		int		  column,
		int		  width,
		int		  align
	)
{
	if (!m_ok)
	{
		QColorGroup cg2 (cg1) ;
		cg2.setColor (QColorGroup::Text, Qt::red) ;
		QListViewItem::paintCell (p, cg2, column, width, align) ;
	}
	else	QListViewItem::paintCell (p, cg1, column, width, align) ;
}



KBTestSuiteResultsDlg::KBTestSuiteResultsDlg ()
	:
	KBDialog (TR("Test Suite Results"), true, "testsuiteresultsdlg")
{
	RKVBox	*lay	= new RKVBox (this) ;
	lay->setTracking () ;

	RKHBox	*top	= new RKHBox (lay ) ;
	new KBSidePanel (top, caption()) ;

	m_results	= new RKListView (top)	;
	m_results->addColumn	(TR("Server"  ), 50 ) ;
	m_results->addColumn	(TR("Location"), 200) ;
	m_results->addColumn	(TR("Line"    ), 50 ) ;
	m_results->addColumn	(TR("Lang."   ), 100) ;
	m_results->addColumn	(TR("Result"  ), 50 ) ;
	m_results->addColumn	(TR("Message" ), 100) ;
	m_results->setColumnWidthMode (0, QListView::Maximum) ;
	m_results->setColumnWidthMode (1, QListView::Maximum) ;
	m_results->setColumnWidthMode (2, QListView::Maximum) ;
	m_results->setColumnWidthMode (3, QListView::Maximum) ;
	m_results->setColumnWidthMode (4, QListView::Maximum) ;
	m_results->setColumnWidthMode (5, QListView::Maximum) ;
	m_results->setResizeMode      (QListView::LastColumn) ;

	RKHBox	*btns	= new RKHBox (lay) ;
	btns->addFiller  () ;
	new RKPushButton (btns, "ok") ;
}

void	KBTestSuiteResultsDlg::addResults
	(	const KBScriptTestResultList	*results
	)
{
	QListViewItem *last = 0 ;

	if (results != 0)
		for (uint idx = 0 ; idx < results->count() ; idx += 1)
		{
			const KBScriptTestResult &result = (*results)[idx] ;
			QStringList bits = QStringList::split (':', result.m_location, true) ;

			last	= new KBTestSuiteResultItem
				  (	m_results,
					last,
					bits[1],
					bits[2],
					result.m_lineNumber,
					result.m_language,
					result.m_result,
					result.m_message
				  )	;
		}
}
