/***************************************************************************
    file	         : kb_blockproxy.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	<stdio.h>

#include	<kjs/global.h>
#include	<kjs/interpreter.h>
#include	<kjs/lookup.h>
#include	<kjs/object.h>
#include	<kjs/operations.h>
#include	<kjs/types.h>
#include	<kjs/ustring.h>
#include	<kjs/value.h>


#include	"kb_classes.h"
#include	"kb_formblock.h"

#include	"kb_idbase.h"
#include	"kb_objectproxy.h"
#include	"kb_itemproxy.h"
#include	"kb_blockproxy.h"
#include	"kb_kjsregister.h"

#include	"kb_kjsscript.h"


#define		__DEBUG_BLOCKPROXY	(0)
#if		__DEBUG_BLOCKPROXY
#include	"kb_dprintf.h"
#endif


/*  KBBlockProxy							*/
/*  KBBlockProxy: Constructor for rekall block proxy			*/
/*  interp	: KJS::Interpreter *	: Intepreter object		*/
/*  block	: KBBlock *		: Rekall block			*/
/*  (returns)	: KBBlockProxy		:				*/

KBBlockProxy::KBBlockProxy
	(	KBKJSInterpreter	*interp,
		KBBlock			*block
	)
	:
	KBItemProxy	(interp, block),
	m_block		(block)
{
}

/*  KBBlockProxy							*/
/* ~KBBlockProxy	: Destructor for rekall block proxy		*/
/*  (returns)	:		:					*/

KBBlockProxy::~KBBlockProxy ()
{
}

/*  KBBlockProxy							*/
/*  addBindings	: Add proxy bindings to specified object		*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  object	: KJS::Object &		: Object to which to add	*/
/*  (returns)	: void			:				*/

void	KBBlockProxy::addBindings
	(	KJS::ExecState	*exec,
		KJS::Object	&object
	)
{
	static	KBMethodSpec methods	[] =
	{
	{	"gotoQueryRow",		MethodImp::id_gotoQueryRow,	0	},
	{	"doAction",		MethodImp::id_doAction,		0	},
	{	"firstRecord",		MethodImp::id_firstRecord,	0	},
	{	"previousRecord",	MethodImp::id_previousRecord,	0	},
	{	"nextRecord",		MethodImp::id_nextRecord,	0	},
	{	"lastRecord",		MethodImp::id_lastRecord,	0	},
	{	"addRecord",		MethodImp::id_addRecord,	0	},
	{	"saveRecord",		MethodImp::id_saveRecord,	0	},
	{	"deleteRecord",		MethodImp::id_deleteRecord,	0	},
	{	"startQuery",		MethodImp::id_startQuery,	0	},
	{	"executeQuery",		MethodImp::id_executeQuery,	0	},
	{	"cancelQuery",		MethodImp::id_cancelQuery,	0	},
	{	"getNumRows",		MethodImp::id_getNumRows,	0	},
	{	"getQueryRow",		MethodImp::id_getQueryRow,	0	},
	{	"overLimit",		MethodImp::id_overLimit,	0	},
	{	"isInQuery",		MethodImp::id_isInQuery,	0	},
	{	"getRowValue",		MethodImp::id_getRowValue,	0	},
	{	"setRowValue",		MethodImp::id_setRowValue,	0	},
	{	"invalidControls",	MethodImp::id_invalidControls,	0	},
	{	"changedControls",	MethodImp::id_changedControls,	0	},
	{	"setUserFilter",	MethodImp::id_setUserFilter,	0	},
	{	"setUserSorting",	MethodImp::id_setUserSorting,	0	},
	{	"reload",		MethodImp::id_reload,		0	},
	{	"saveAsNew",		MethodImp::id_saveAsNew,	0	},
	{	"saveUnchecked",	MethodImp::id_saveUnchecked,	0	},
	{	"gotoQueryRowByKey",	MethodImp::id_gotoQueryRowByKey,0	},
	{	0,			0,				0	}
	}	;

	KBItemProxy::addBindings (exec, object) ;

	for (KBMethodSpec *ms = &methods[0] ; ms->m_method != 0 ; ms += 1)
		object.put
		(	exec,
			KJS::Identifier(ms->m_method),
			KJS::Object    (new MethodImp(ms, this))
		)	;
}

/*  KBBlockProxy							*/
/*  get		: Get property value					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  property	: KJS::Identifier &	: Property name			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBBlockProxy::get
	(	KJS::ExecState		*exec,
		const KJS::Identifier	&property
	)
	const
{
	QString	p = property.qstring() ;

	DPRINTF
	((	"KBBlockProxy::get: [%s][%s] cs=%d\n",
		(cchar *)m_object->getName(),
		(cchar *)p,
		m_interp->clientSide()
	))	;

	return	KBItemProxy::get (exec, property) ;
}

/*  KBBlockProxy							*/
/*  put		: Set property value					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  property	: KJS::Identifier &	: Property name			*/
/*  value	: const KJS::Value &	: Value				*/
/*  attr	: int			:				*/
/*  (returns)	: void			:				*/

void	KBBlockProxy::put
	(	KJS::ExecState		*exec,
		const KJS::Identifier	&property,
		const KJS::Value	&value,
		int			attr
	)
{
	KJS::ObjectImp::put (exec, property, value, attr) ;
}


/*  ------------------------------------------------------------------  */

/*  KBBlockProxy							*/
/*  MethodImp	: 							*/
/*  MethodImp	: Object implementation					*/
/*  spec	: KBMethodSpec * : SPecification			*/
/*  object	: KBBlockProxy * : Associated proxy			*/
/*  (return)	: MethodImp	 :					*/

KBBlockProxy::MethodImp::MethodImp
	(	KBMethodSpec		*spec,
		const KBBlockProxy	*block
	)
	:
	KBItemProxy::MethodImp(spec, block),
	m_block		      (block)
{
}


/*  MethodImp								*/
/*  callBase	: Invoke a method					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  obj		: KJS::Object &		: Apparently "self"		*/
/*  args	: const KJS::List	: Argument list			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBBlockProxy::MethodImp::callBase
	(	KJS::ExecState		*exec,
		KJS::Object		&object,
		const KJS::List		&args
	)
{
	uint		arg0	= args.at(0).toInteger(exec) ;
	uint		arg1	= args.at(1).toInteger(exec) ;
	KBFormBlock	*fblk	= m_block->m_block->isFormBlock() ;

	if (fblk == 0)
		return	KBItemProxy::MethodImp::callBase (exec, object, args) ;

	switch (m_methodSpec->m_id)
	{
		case id_gotoQueryRow	:
			fblk->gotoQRow (arg0) ;
			break	;

		case id_doAction	:
			fblk->doAction ((KB::Action)arg0) ;
			break	;

		case id_firstRecord	:
			fblk->doAction (KB::First   ) ;
			break	;

		case id_previousRecord	:
			fblk->doAction (KB::Previous) ;
			break	;

		case id_nextRecord	:
			fblk->doAction (KB::Next    ) ;
			break	;

		case id_lastRecord	:
			fblk->doAction (KB::Last    ) ;
			break	;

		case id_addRecord	:
			fblk->doAction (KB::Add     ) ;
			break	;

		case id_saveRecord	:
			fblk->doAction (KB::Save    ) ;
			break	;

		case id_deleteRecord	:
			fblk->doAction (KB::Delete  ) ;
			break	;

		case id_startQuery	:
			fblk->doAction (KB::Query   ) ;
			break	;

		case id_executeQuery	:
			fblk->doAction (KB::Execute ) ;
			break	;

		case id_cancelQuery	:
			fblk->doAction (KB::Cancel  ) ;
			break	;

		case id_getNumRows	:
			return	KJS::Number (fblk->getNumRows()) ;

		case id_getQueryRow	:
			return	KJS::Number (fblk->getCurQRow()) ;

		case id_overLimit	:
			return	KJS::Number (fblk->overLimit ()) ;

		case id_isInQuery	:
			return	KJS::Number (fblk->isInQuery ()) ;

		case id_getRowValue	:
			return	KBObjectProxy::fromKBValue
				(	exec,
					fblk->getRowValue
					(	kjsStringArg (exec, args, 0),
						kjsNumberArg (exec, args, 0)
					)
				)	;

		case id_setRowValue	:
			fblk->setRowValue
			(	kjsStringArg (exec, args, 0),
				kjsNumberArg (exec, args, 1),
				kjsNumberArg (exec, args, 2)
			)	;
			return	KJS::Number(0) ;

		case id_invalidControls	:
		{
			KJS::List	  items	  ;
			QPtrList<KBItem>  invalid ;

			bool	anySet	= fblk->invalidControls (arg0, invalid, true) ;

			/* If any control in the record were set, or	*/
			/* we require that the row must be filled in,	*/
			/* then append any invalid items to the		*/
			/* results list.				*/
			if (anySet || arg1)
				for (uint idx = 0 ; idx < invalid.count() ; idx += 1)
					items.append (fromKBNode (exec, invalid.at(idx))) ;

			return	KJS::Object::dynamicCast
				(	exec->interpreter()->builtinArray().construct
					(	exec,
						items
				)	)	;
		}

		case id_changedControls	:
		{
			KJS::List	  items	  ;
			QPtrList<KBItem>  changed ;

			fblk->changedControls (getCurQRow(arg0), changed, true) ;

			for (uint idx = 0 ; idx < changed.count() ; idx += 1)
				items.append (fromKBNode (exec, changed.at(idx))) ;

			return	KJS::Object::dynamicCast
				(	exec->interpreter()->builtinArray().construct
					(	exec,
						items
				)	)	;
		}

		case id_setUserFilter	:
			fblk->setUserFilter  (kjsStringArg (exec, args, 0)) ;
			break	;

		case id_setUserSorting	:
			fblk->setUserSorting (kjsStringArg (exec, args, 0)) ;
			break	;

		case id_reload		:
			fblk->doAction (KB::Reload) ;
			break	;

		case id_saveAsNew	:
			fblk->doAction (KB::SaveAsNew) ;
			break	;

		case id_saveUnchecked	:
			fblk->doAction (KB::SaveUnchecked) ;
			break	;

		case id_gotoQueryRowByKey :
			return	KJS::Number
				(	fblk->gotoQRow (KBValue (kjsStringArg (exec, args, 0)))
				)	;

		default	:
			break	;
	}

	return	KBItemProxy::MethodImp::callBase (exec, object, args) ;
}

/*  MethodImp								*/
/*  call	: Invoke a method					*/
/*  exec	: KJS::ExecState *	: Execution state		*/
/*  obj		: KJS::Object &		: Apparently "self"		*/
/*  args	: const KJS::List	: Argument list			*/
/*  (returns)	: KJS::Value		: Result			*/

KJS::Value
	KBBlockProxy::MethodImp::call
	(	KJS::ExecState		*exec,
		KJS::Object		&object,
		const KJS::List		&args
	)
{
	KJS::Value	error	= checkArgs (exec, args) ;
	if (error.isValid()) return error ;

	return	callBase (exec, object, args) ;
}


KBObjectProxy
	*makeBlockProxy
	(	KBKJSInterpreter	*interp,
		KBNode			*node
	)
{
	return	new KBBlockProxy (interp, (KBBlock *)node) ;
}

static	const char	*blockAlias[] =
{	
	"KBFormBlock",
	"KBReportBlock",
	"KBFormSubBlock",
	"KBReportSubBlock",
	0
}	;

void	registerKBBlock	()
{
	registerClass	("KBBlock", makeBlockProxy, blockAlias) ;
}

