/**************************************************************************
    file	         : kb_text.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<kjs/global.h>
#include	<kjs/interpreter.h>
#include	<kjs/lookup.h>
#include	<kjs/object.h>
#include	<kjs/operations.h>
#include	<kjs/types.h>
#include	<kjs/ustring.h>
#include	<kjs/value.h>


#include	"kb_classes.h"
#include	"kb_script.h"

#include	"kb_kjsscript.h"
#include	"kb_kjdebug.h"

#include	"kb_test.h"
#include	"kb_test.lut"

#include	"tk_messagebox.h"

#define		__DEBUG_TEST	(0)
#if		__DEBUG_TEST
#include	"kb_dprintf.h"
#endif


static	KBScriptIF::TestMode	testMode	;
static	KBScriptTestResultList	*testResults	;
static	bool			testException	;

void	kjsTestSetTestMode
	(	KBScriptIF::TestMode	mode
	)
{
	if (testResults == 0)
		testResults = new KBScriptTestResultList ;

	testMode = mode	      ;
	testResults->clear () ;
}

const KBScriptTestResultList *kjsTestGetTestResults ()
{
	return	testResults	;
}

void	kjsTestClearTestException ()
{
	testException = false	;
}

bool	kjsTestHadTestException ()
{
	return	testException	;
}

/*  ------------------------------------------------------------------	*/

const	ClassInfo	RekallTestObjectImp::info =
			{
				"RekallTest",
				0,
				&rekallTestTable,
				0
			}	;


#if	0
@begin rekallTestTable 21
  test	          RekallTestObjectImp::test		DontEnum|Function 1
@end
#endif

/*  test								*/
/*		: Basic test assertion					*/
/*  self	: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: 					*/

static	Value	test
	(	ExecState	*exec,
		const List	&args
	)
{
	bool		ok	= kjsBooleanArg (exec, args, 0) ;
	QString		message	= kjsStringArg  (exec, args, 1) ;


	KBKJSDebugger	*d 	= KBKJSDebugger::self () ;
	int	 	id	= d->lastSourceID     () ;
	uint		lineno	= d->lastLine         () ;

	KBKJSScriptCode *code	= KBKJSScriptCode::lookup (id) ;
	QString		name	;

	if (code != 0)
		name	= code->m_location.ident() ;
	else	name	= ":Unknown:Unknown:"	   ;

	if (!ok)
	{
		if (testResults != 0)
			testResults->append
			(	KBScriptTestResult
				(	name,
					lineno,
					KBScriptTestResult::testFailed,
					message,
					"kjs"
			)	)	;

		if (testMode == KBScriptIF::testSuite)
		{
			KJS::Value err	= KJS::Error::create
					  (	exec,
						KJS::GeneralError,
						QString	("Test suite failure")
					  )	;
			exec->setException(err)	;
			testException	= true	;
			return	err ;
		}

		QStringList bits = QStringList::split (':', name, true) ;
		TKMessageBox::sorry
		(	0,
			QString	(TR("Server: %1\nLocation: %2\nLine: %3\nMessage: %4"))
				.arg(bits[1])
				.arg(bits[2])
				.arg(lineno)
				.arg(message),
			TR("Test failure")
		)	;

		return	KJS::Boolean(false) ;
	}

	if (testResults != 0)
		testResults->append
		(	KBScriptTestResult
			(	name,
				lineno,
				KBScriptTestResult::testOK,
				message,
				"kjs"
		)	)	;

	return	KJS::Boolean(true) ;
}

/*  ------------------------------------------------------------------	*/

RekallTestObjectImp::RekallTestObjectImp
	(	ExecState		*,
		ObjectPrototypeImp	*objProto
	)
	:
	ObjectImp (Object(objProto))
{
	DPRINTF
	((	"RekallTestObjectImp::RekallTestObjectImp: called\n"
	))	;
}


Value	RekallTestObjectImp::get
	(	ExecState	 *exec,
		const Identifier &propertyName
	)
	const
{
	DPRINTF
	((	"RekallTestObjectImp::get\n"
	))	;

	return	lookupGet<RekallTestFunctionImp, RekallTestObjectImp, ObjectImp>
			 (	exec,
				propertyName,
				&rekallTestTable,
				this
			 )	;
}


Value	RekallTestObjectImp::getValueProperty
	(	ExecState	*,
		int		token
	)
	const
{
	switch (token)
	{
		default	:
			break	;
	}

	return	Number(-1)	;
}


RekallTestFunctionImp::RekallTestFunctionImp
	(	ExecState	*exec,
		int		id,
		int		length
	)
	:
	InternalFunctionImp
	(
		(FunctionPrototypeImp*)(exec->interpreter()->builtinFunctionPrototype().imp())
	),
	m_id	(id)
{
	DPRINTF
	((	"RekallTestFunctionImp::RekallTestFunctionImp: called\n"
	))	;

	Value	protect (this) ;
	put	(exec, "length", Number(length), DontDelete|ReadOnly|DontEnum) ;
}

bool	RekallTestFunctionImp::implementsCall () const
{
	return	true	;
}

Value	RekallTestFunctionImp::call
	(	ExecState	*exec,
		Object		&,
		const List	&args
	)
{
	DPRINTF
	((	"RekallTestFunctionImp::call: m_id=%d\n",
		m_id
	))	;

	switch (m_id)
	{
		case RekallTestObjectImp::test:
			return	test (exec, args) ;

		default	:

			break	;
	}

	return	Number(-1) ;
}
