/**************************************************************************
    file	         : kb_basic.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/

#include	"kb_python.h"

#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_location.h"
#include	"kb_callback.h"
#include	"kb_prompt.h"
#include	"kb_pyscript.h"
#include	"kb_propdict.h"
#include	"kb_prompt.h"
#include	"kb_databuffer.h"
#include	"kb_base64.h"
#include	"kb_blowfish.h"
#include	"kb_object.h"
#include	"kb_docroot.h"
#include	"kb_helper.h"
#include	"kb_pybase.h"
#include	"kb_pyvalue.h"

#include	"tk_messagebox.h"

#include	"tkc_pydebugbase.h"

extern	QString 	getPythonString	(PyObject *) ;
extern	KBCallback	*kbCallback	;


/*  kbPYMessageBox							*/
/*		: Show simple message box				*/
/*  self	: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: 					*/

static	PyObject *kbPYMessageBox
	(	 PyObject	*,
		 PyObject	*args
	)
{
 	PyObject 	*message;
 	PyObject 	*caption = 0;
 	QString		qMessage;
 	QString		qCaption;
 	bool		error = false;

	if (!PyArg_ParseTuple (args, __C("O|O"), &message, &caption))
		return	0	;

	if (caption == 0) qCaption = "Database" ;
	else 		  qCaption = kb_pyStringToQString(caption, error);
	if (error)
		return 0;
	
	qMessage = kb_pyStringToQString(message, error);
	if (error)
		return 0;

	TKMessageBox::information
	(	0,
		qMessage,
		qCaption
	)	;

	Py_INCREF(Py_None) ;
	return 	  Py_None  ;
}


/*  kbPYQueryBox: Show simple yes/no query box				*/
/*  self	: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: True if yes				*/

static	PyObject *kbPYQueryBox
	(	 PyObject	*,
		 PyObject	*args
	)
{
	PyObject *message;
	PyObject *caption = 0;
	QString	 qMessage;
	QString	 qCaption;
	bool	 error = false;

	if (!PyArg_ParseTuple (args, __C("O|O"), &message, &caption))
		return	0 ;

	if (caption == 0) qCaption = "Database" ;
	else		  qCaption = kb_pyStringToQString(caption, error);
	if (error)
		return	0 ;
	
	qMessage = kb_pyStringToQString(message, error);
	if (error)
		return 0;

	bool	yes	= TKMessageBox::questionYesNo
			  (	0,
				qMessage,
				qCaption
			  )
			  == TKMessageBox::Yes ;

	return	PyInt_FromLong (yes) ;
}


/*  kbPYPromptBox							*/
/*		: Show simple prompt					*/
/*  self	: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: 					*/

static	PyObject *kbPYPromptBox
	(	 PyObject	*,
		 PyObject	*args
	)
{
	PyObject	*message;
	PyObject	*caption	= 0 ;
	PyObject	*defval		= 0 ;
	
	QString		qMessage	;
	QString		qDefval		;
	QString		qCaption	;
	QString		value		;
	bool		error = false	;

	if (!PyArg_ParseTuple (args, __C("O|OO"), &message, &defval, &caption))
		return	0	;

	if (caption == 0) qCaption = "Database" ;
        else		  qCaption = kb_pyStringToQString(caption, error);
        if (error)
        	return 0;
        
	if (defval != 0) 
	{
		value =	kb_pyStringToQString(defval, error);
		if (error)
			return 0;
	}
	
	qMessage = kb_pyStringToQString(message, error);
	if (error)
		return 0;
	

	if (!doPrompt (qCaption, qMessage, value))
	{	Py_INCREF(Py_None) ;
		return	  Py_None  ;
	}

	return	kb_qStringToPyString(value);
}

/*  kbPYChoiceBox							*/
/*		: Show simple prompt					*/
/*  self	: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: 					*/

static	PyObject *kbPYChoiceBox
	(	 PyObject	*,
		 PyObject	*args
	)
{
	PyObject	*message	= 0 ;
	PyObject	*choices	= 0 ;
	PyObject	*caption	= 0 ;
	QStringList	cList		;
	QString		resVal		;
	QString		qMessage	;
	QString		qCaption	;
	bool		error = false	;

	if (!PyArg_ParseTuple (args, __C("OO!|O"), &message, &PyList_Type, &choices, &caption))
		return	0	;

	qMessage = kb_pyStringToQString(message, error);
	if (error)
		return 0;
	if (caption == 0) qCaption = "Database" ;
	else		  qCaption = kb_pyStringToQString(caption, error);
	if (error)
		return 0;

	for (int idx = 0 ; idx < PyList_Size(choices) ; idx += 1)
	{
		cList.append (kb_pyStringToQString(PyList_GetItem(choices, idx), error)) ;
		if (error)
			return 0;
	}

	KBChoiceDlg dDlg (qCaption, qMessage, cList, resVal) ;
	if (!dDlg.exec())
	{	Py_INCREF(Py_None) ;
		return	  Py_None  ;
	}

	return  kb_qStringToPyString(resVal);
}

/*  kbPYLogscript							*/
/*		: Log message to script log window			*/
/*  self	: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: 					*/

static	PyObject *kbPYLogscript
	(	 PyObject	*,
		 PyObject	*args
	)
{
	PyObject	*text	= 0 ;
	QString		qText;
	bool		error;

	if (!PyArg_ParseTuple (args, __C("O"), &text))
		return	0	;
	
	qText = kb_pyStringToQString(text, error);
	if (error)
		return 0;

	kbCallback->logScript (qText, true) ;

	Py_INCREF(Py_None) ;
	return	  Py_None  ;
}

/*  kbPYGetDictEntry							*/
/*		: Get dictionary entry					*/
/*  self	: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: 					*/

static	PyObject *kbPYGetDictEntry
	(	 PyObject	*,
		 PyObject	*args
	)
{
	cchar		*dict	= 0 ;
	cchar		*elem	= 0 ;
	cchar		*attr	= 0 ;
	KBPropDict	*dictionary ;	


	// xxx the dict below is local to this function. Hence its values
	// (i.e., the property dicts) will always be empty. Or am I missing
	// sommething?

	static	QDict<KBPropDict>	dictionaries ;

	fprintf
	(	stderr,
		"kbPYGetDictEntry: called\n"
	)	;

	if (!PyArg_ParseTuple (args, __C("sss"), &dict, &elem, &attr))
		return	0	;

	fprintf
	(	stderr,
		"kbPYGetDictEntry: called [%s][%s][%s]\n",
		dict,
		elem,
		attr
	)	;

	if ((dictionary = dictionaries.find (dict)) == 0)
	{
		dictionary = new KBPropDict (dict) ;
		dictionaries.insert (dict, dictionary) ;
	}

	if (dictionary == 0) return PyString_FromString("") ;

	KBPropDictEntry *entry = dictionary->getEntry (elem, attr) ;
	if (entry == 0) return PyString_FromString("") ;

	return  kb_qStringToPyString(entry->m_descrip);
	
}


/*  kbPyDebug								*/
/* 		: Enter debugger					*/
/*  self	: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: 					*/

static	PyObject *kbPyDebug
	(	 PyObject	*,
		 PyObject	*args
	)
{
	TKCPyDebugBase	*deb	= TKCPyDebugBase::self() ;

	if (deb == 0)
	{	
		Py_INCREF(Py_None) ;
		return 	  Py_None  ;
	}

	cchar	*msg	= 0	;
	PyArg_ParseTuple (args, __C("s"), &msg) ;

	return	deb->debugHook (PyEval_GetFrame(), msg) ;
}

/*  kbPyWebMode	: Get web execution mode				*/
/*  self	: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: 					*/

static	PyObject *kbPyWebMode
	(	 PyObject	*,
		 PyObject	*args
	)
{
	/* This is mode zero, Python executing inside Rekall itself.	*/
	return	PyInt_FromLong(0) ;
}

static	PyObject *kbEncryptDict
	(	PyObject	*,
		PyObject	*args
	)
{
	PyObject	*pyDict		;
	const char	*blowKey	;

	if (!PyArg_ParseTuple (args, __C("Os"), &pyDict, &blowKey))
		return	0	;
	if (!PyDict_Check (pyDict))
		return	0	;

	QStringList	pairs	;
	PyObject	*key	;
	PyObject	*value	;
	int		pos	= 0 ;

	//xxx use pickle or marshal to create the data representation instead
	//xxx of a simple string representation?
	//xxx if so, add as a new function, 'cos I use this! mike
	while (PyDict_Next (pyDict, &pos, &key, &value))
		pairs.append (getPythonString (key) + "=" + getPythonString (value)) ;

	KBDataBuffer	text	;
	KBDataBuffer	base64	;

	text.append (pairs.join(";")) ;
	while ((text.length() % 8) != 0) text.append (';') ;
	kbBlowfishEncipher (blowKey, (unsigned char *)text.data(), text.length()) ;
	kbB64Encode	   ((unsigned char *)text.data(), text.length(), base64)  ;

	return	PyString_FromString (base64.data()) ;
}

static	PyObject *kbDecryptDict
	(	PyObject	*,
		PyObject	*args
	)
{
	const char	*base64		;
	const char	*blowKey	;

	if (!PyArg_ParseTuple (args, __C("ss"), &base64, &blowKey))
		return	0	;

	KBDataBuffer	text		;
	kbB64Decode	   ((const unsigned char *)base64, strlen(base64), text) ;
	kbBlowfishDecipher (blowKey, (unsigned char *)text.data(), text.length()) ;

	PyObject	*pyDict	= PyDict_New () ;
	if (pyDict == 0)
		return 0;
	QStringList	pairs	= QStringList::split (';', text.data()) ;
	for (uint idx = 0 ; idx < pairs.count() ; idx += 1)
	{
		QStringList pair = QStringList::split ('=', pairs[idx]) ;
		if (pair.count() == 2)
		{
			PyObject *pyVal = PyString_FromString(__C(pair[1]));
			if (pyVal == 0)
			{
				Py_DECREF(pyDict);
				return 0;
			}
			if (-1 == PyDict_SetItemString(	pyDict,
							__C(pair[0]),
							pyVal))
			{
				Py_DECREF(pyVal);
				Py_DECREF(pyDict);
				return 0;
			}
			Py_DECREF(pyVal);
		}
	}

	return	pyDict	;
}

/*  kbPYHelperPopup							*/
/*		: Display asynchronous popup helper			*/
/*  self	: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: 					*/

static	PyObject *kbPYHelperPopup
	(	 PyObject	*,
		 PyObject	*args
	)
{
 	PyObject *helper  ;
 	PyObject *value	  ;
	PyObject *target  ;
	PyObject *slot    ;
	PyObject *format  = 0 ;

 	QString	 qHelper  ;
 	QString	 qValue   ;
 	QString	 qSlot	  ;
	QString	 qFormat  ;

	PyKBBase *kTarget ;
 	bool	 error	  = false ;

	if (!PyArg_ParseTuple (args, __C("OOOO|O"), &helper, &value, &target, &slot, &format))
		  return 0	;

	qHelper = kb_pyStringToQString (helper, error) ;
	if (error) return 0	;
	qValue  = kb_pyStringToQString (value,  error) ;
	if (error) return 0	;
	qSlot   = kb_pyStringToQString (slot,   error) ;
	if (error) return 0	;

	if (format != 0)
	{	qFormat   = kb_pyStringToQString (format, error) ;
		if (error) return 0 ;
	}

	cchar	*errMsg	;
	kTarget	= PyKBBase::getPyBaseFromPyInst (target, PyKBBase::m_object, errMsg) ;
	if (kTarget == 0)
	{
		PyErr_SetString (PyKBRekallError, errMsg) ;
		return	0	;
	}

	KBObject *object = (KBObject *)kTarget->m_kbObject ;

	API_CALL (object, "kbPYHelperPopup") ;
	KBHelperPopup::run
		(	qHelper,
			qValue,
			object->getDocRoot()->getLocation(),
			object,
			qSlot,
			qFormat
		) ;
	API_DONE  ;

	return 	  Py_None  ;
}


static	PyMethodDef	basicFuncs[] =
{
{	__C("messageBox"),	kbPYMessageBox,		METH_VARARGS,	0	},
{	__C("queryBox"),	kbPYQueryBox,		METH_VARARGS,	0	},
{	__C("promptBox"),	kbPYPromptBox,		METH_VARARGS,	0	},
{	__C("choiceBox"),	kbPYChoiceBox,		METH_VARARGS,	0	},
{	__C("logscript"),	kbPYLogscript,		METH_VARARGS,	0	},
{	__C("getDictEntry"),	kbPYGetDictEntry,	METH_VARARGS,	0	},
{	__C("debug"),		kbPyDebug,		METH_VARARGS,	0	},
{	__C("webMode"),		kbPyWebMode,		METH_VARARGS,	0	},
{	__C("encryptDict"),	kbEncryptDict,		METH_VARARGS,	0	},
{	__C("decryptDict"),	kbDecryptDict,		METH_VARARGS,	0	},
{	__C("showPopupHelper"),	kbPYHelperPopup,	METH_VARARGS,	0	},
{	0,			0,			0,		0	}
}	;

void	initPYBasic ()
{
	Py_InitModule (__C("RekallMain"), basicFuncs) ;
}

