/**************************************************************************
    file	         : kb_text.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	"kb_python.h"

#include	"kb_classes.h"
#include	"kb_pyscript.h"
#include	"kb_pybase.h"
#include	"kb_pyvalue.h"

#include	"tk_messagebox.h"
#include	"tkc_pydebugbase.h"


static	KBScriptIF::TestMode	testMode	;
static	KBScriptTestResultList	*testResults	;


/*  pyTestSetTestMode							*/
/*		: Set required test (suite) mode			*/
/*  mode	: KBScriptIF::TestMode	: Required mode			*/
/*  (returns)	: void			:				*/

void	pyTestSetTestMode
	(	KBScriptIF::TestMode	mode
	)
{
	if (testResults == 0)
		testResults = new KBScriptTestResultList ;

	testMode = mode	      ;
	testResults->clear () ;
}

const KBScriptTestResultList *pyTestGetTestResults ()
{
	return	testResults	;
}


/*  kbPYTest								*/
/*		: Basic test assertion					*/
/*  self	: PyObject *	:					*/
/*  args	: PyObject *	: Arguments				*/
/*  (returns)	: PyObject *	: 					*/

static	PyObject *kbPYTest
	(	 PyObject	*,
		 PyObject	*args
	)
{
 	int	 	value	;
	PyObject 	*text	= 0 ;
	QString		qText	;

	if (!PyArg_ParseTuple (args, __C("i|O"), &value, &text))
		return	0	;

	if (text != 0)
	{
		bool	error	;
		qText = kb_pyStringToQString (text, error) ;
	}

	extern	QString	getPythonString (PyObject *) ;

	PyFrameObject *frame  = PyThreadState_GET()->frame ;
	PyCodeObject  *caller = frame->f_code		   ;
	int	      lineno  = frame->f_lineno		   ;
	QString	      name    = getPythonString (caller->co_filename) ;

	if (!value)
	{
		if (testResults != 0)
			testResults->append
			(	KBScriptTestResult
				(	name,
					lineno,
					KBScriptTestResult::testFailed,
					qText,
					"py"
			)	)	;

		if (testMode == KBScriptIF::testSuite)
		{
			PyErr_SetString (PyKBRekallTest, "Test suite failure") ;
			return	0 ;
		}

		QStringList bits = QStringList::split (':', name, true) ;
		TKMessageBox::sorry
		(	0,
			QString	(TR("Server: %1\nLocation: %2\nLine: %3\nMessage: %4"))
				.arg(bits[1])
				.arg(bits[2])
				.arg(lineno)
				.arg(qText),
			TR("Test failure")
		)	;

		return	PyInt_FromLong (0) ;
	}

	if (testResults != 0)
		testResults->append
		(	KBScriptTestResult
			(	name,
				lineno,
				KBScriptTestResult::testOK,
				qText,
				"py"
		)	)	;

	return	PyInt_FromLong (1) ;
}

static	PyMethodDef	testFuncs[] =
{
{	__C("test"),	kbPYTest,		METH_VARARGS,	0	},
{	0,			0,			0,		0	}
}	;

void	initPYTest ()
{
	Py_InitModule (__C("RekallTest"), testFuncs) ;
}

