/***************************************************************************
    file	         : testdriver.cpp
    copyright            : (C)	1999,2000,2001,2002,2003,2004,2005
				 by Mike Richardson
			   (C)	2001,2002,2003,2004,2005
				by John Dean
    license              : This file is released under the terms of
                           the GNU General Public License, version 2. The
                           copyright holders retain the right to release
                           this code under diffenent non-exclusive licences.
    email                : mike@quaking.demon.co.uk                                     
 ***************************************************************************/


#include	<qmap.h>
#include	<qstring.h>
#include	<qdict.h>
#include	<qintdict.h>
#include	<qregexp.h>
#include	<qdir.h>
#include	<qdatetime.h>


#if		defined(__WIN32__)
#define		EXPORT_API	__declspec(dllexport)
#endif
#if		! defined(__WIN32__)
#define		EXPORT_API
#endif


#include	"kb_classes.h"
#include	"kb_type.h"
#include	"kb_value.h"
#include	"kb_databuffer.h"
#include	"kb_database.h"
#include	"kb_serverinfo.h"
#include	"kb_libloader.h"
#include	"kb_locator.h"
#include	"kb_dom.h"

struct	TestDriverTypeMap
{	int		ident		;	/* TestDriver identifier	*/
	KB::IType	itype		;	/* Internal type	*/
	char		mtype	[16]	;	/* TestDriver type name	*/
	uint		flags		;	/* Various flags	*/
}	;



#define	FIELD_TYPE_UNSET	(-1)

static	TestDriverTypeMap	typeMap[] =
{
/*	ident			itype		mtype		flags		*/
{	0,			KB::ITFixed,	"Integer",	0			},
{	1,			KB::ITFloat,	"Float",	FF_LENGTH|FF_PREC	},

{	2,			KB::ITDate,	"Date",		0			},
{	3,		 	KB::ITTime,	"Time",		0			},
{	4,		 	KB::ITDateTime,	"DateTime",	0			},

{	5,			KB::ITString,	"Char",		FF_LENGTH		},
{	6,			KB::ITString,	"VarChar",	FF_LENGTH		},

{	7,			KB::ITBinary,	"Binary",	0			}
}	;




/*  KBTestDriverType								*/
/*  -----------								*/

class	KBTestDriverType : public KBType
{
	TestDriverTypeMap	*m_typeInfo	;

public	:

	KBTestDriverType  (TestDriverTypeMap *, uint, uint, bool) ;

	virtual	bool    isValid      (const QString &, KBError  &, const QString & = QString::null) ;
	virtual	void	getQueryText (KBDataArray   *, KBShared *, KBDataBuffer  &) ;
}	;



/*  KBTestDriver								*/
/*  -------								*/
/*  Implementation class for insertface to the TestDriver database		*/

class	KBTestDriver : public KBServer
{
	bool			m_mapExpressions	;

	bool			doListTables 	(KBTableDetailsList &, uint) ;

	virtual	bool	 	doConnect    	(KBServerInfo  *) ;
	virtual	bool		doListFields 	(KBTableSpec   &) ;
	virtual	bool		doCreateTable	(KBTableSpec   &,  bool, bool = false) ;
	virtual	bool		doRenameTable	(cchar *, cchar *, bool) ;
	virtual	bool		doDropTable  	(cchar *,	   bool) ;

public	:

	KBTestDriver	() ;
	KBTestDriver	(const QMap<QString,QString> &) ;
virtual~KBTestDriver	() ;


	virtual	KBSQLSelect	*qrySelect 	(bool, const QString &, bool) ;
	virtual	KBSQLUpdate	*qryUpdate 	(bool, const QString &, const QString &) ;
	virtual	KBSQLInsert	*qryInsert 	(bool, const QString &, const QString &) ;
	virtual	KBSQLDelete	*qryDelete 	(bool, const QString &, const QString &) ;

	virtual	bool		command		(bool, const QString &, uint, KBValue *, KBSQLSelect ** = 0) ;

	virtual	QString		mapExpression	(const QString &) ;
	virtual	QString		listTypes	() ;
	virtual	bool		tableExists	(const QString &, bool &) ;
}	;


/*  KBTestDriverQrySelect							*/
/*  ----------------							*/
/*  Implementation class for select queries on the TestDriver database	*/

class	KBTestDriverQrySelect : public KBSQLSelect
{
	KBTestDriver		*m_server	;
	QString			m_query		;
	QString			m_table		;
	QStringList		m_columns	;
	QValueList<QDomElement>	m_rows		;
//	QDomNodeList		m_rows		;

	uint			m_crow		;
	ulong			*m_lengths	;

public	:
	KBTestDriverQrySelect (KBTestDriver *, bool, const QString &) ;
virtual~KBTestDriverQrySelect () ;

	virtual	bool	execute		(uint, const KBValue * ) ;
	virtual	KBValue	getField	(uint, uint, KBValue::VTrans) ;
	virtual	QString	getFieldName	(uint) ;
}	;


/*  KBTestDriverQryUpdate							*/
/*  ----------------							*/
/*  Implementation class for update queries on the TestDriver database	*/

class	KBTestDriverQryUpdate : public KBSQLUpdate
{
	KBTestDriver	*m_server	;

public	:
	KBTestDriverQryUpdate (KBTestDriver *, bool, const QString &, const QString &) ;
virtual~KBTestDriverQryUpdate () ;

	virtual	bool	execute	 (uint, const KBValue * ) ;

}	;

/*  KBTestDriverQryInsert							*/
/*  ----------------							*/
/*  Implementation class for insert queries on the TestDriver database	*/

class	KBTestDriverQryInsert : public KBSQLInsert
{
	KBTestDriver		*m_server	;
	QString		m_autocol	;
	KBValue		m_newKey	;

public	:
	KBTestDriverQryInsert (KBTestDriver *, bool, const QString &, const QString &) ;
virtual~KBTestDriverQryInsert () ;

	virtual	bool	execute	  (uint, const KBValue *) ;
	virtual	bool	getNewKey (const QString &, KBValue &, bool) ;
}	;

/*  KBTestDriverQryDelete							*/
/*  ----------------							*/
/*  Implementation class for delete queries on the TestDriver database	*/

class	KBTestDriverQryDelete : public KBSQLDelete
{
	KBTestDriver		*m_server	;

public	:
	KBTestDriverQryDelete (KBTestDriver *, bool, const QString &, const QString &) ;
virtual~KBTestDriverQryDelete () ;

	virtual	bool	execute	 (uint, const KBValue * ) ;
}	;


static	QIntDict<TestDriverTypeMap>	dIdentToType	;
static	QDict	<KBTestDriverType >	typesDict	;

/*  ------------------------------------------------------------------  */

/*  KBTestDriverType							*/
/*  KBTestDriverType							*/
/*		: Constructor for TestDriver type object		*/
/*  typeInfo	: TestDriverTypeMap * : Type information		*/
/*  length	: uint		 : Underlying database length		*/
/*  prec	: uint		 : Underlying database precision	*/
/*  nullOK	: bool		 : True if null is OK			*/
/*  (returns)	: KBTestDriverType	 :				*/

KBTestDriverType::KBTestDriverType
	(	TestDriverTypeMap	*typeInfo,
		uint	  	length,
		uint		prec,
		bool	  	nullOK
	)
	:
	KBType
	(	"Test",
		typeInfo == 0 ? KB::ITUnknown : typeInfo->itype,
		length,
		prec,
		nullOK
	),
	m_typeInfo	(typeInfo)
{
}

/*  KBTestDriverType							*/
/*  isValid	: Test if value is valid for type			*/
/*  value	: const QString & : Value to check			*/
/*  pError	: KBValue &	  : Error return			*/
/*  where	: const QString & : Caller				*/
/*  (returns)	: bool		  : Valid				*/

bool	KBTestDriverType::isValid
	(	const QString	 &value,
		KBError		 &pError,
		const QString	&where
	)
{
	/* *** Specific type checking					*/
	return	KBType::isValid (value, pError, where) ;
}

/*  KBTestDriverType							*/
/*  getQueryText							*/
/*  value	: KBDataArray  * : Raw text of value to convert		*/
/*  d		: KBShared     * : Decoded representation		*/
/*  buffer	: KBDataBuffer & : Results buffer			*/
/*  (returns)	: void		 :					*/

void	KBTestDriverType::getQueryText
	(	KBDataArray	*value,
		KBShared	*d,
		KBDataBuffer	&buffer
	)
{
	/* **** Stuff like binary escaping				*/
	KBType::getQueryText (value, d, buffer) ;
}





/*  ------------------------------------------------------------------  */

/*  KBTestDriver							*/
/*  KBTestDriver: Constructor for TestDriver database connection class	*/
/*  (returns)	: KBServer	:					*/

KBTestDriver::KBTestDriver ()
	:
	KBServer ()
{
	m_mapExpressions	= false	;
}

KBTestDriver::KBTestDriver
	(	const QMap<QString,QString>	&options
	)
	:
	KBServer ()
{
	m_mapExpressions	= options["mapExpressions"].toInt() ;
}

/*  KBTestDriver							*/
/*  ~KBTestDriver	: Destructor for TestDriver database connection class	*/
/*  (returns)	:		:					*/

KBTestDriver::~KBTestDriver ()
{
}

/*  KBTestDriver							*/
/*  doConnect	: Open connection to database				*/
/*  svInfo	: KBServerInfo *: Server information			*/
/*  (returns)	: bool		: Success				*/

bool	KBTestDriver::doConnect
	(	KBServerInfo	*
	)
{
	return	true	;
}

/*  KBTestDriver							*/
/*  qrySelect	: Open a select query					*/
/*  data	: bool		  : Query for data			*/
/*  select	: const QString&: Select query				*/
/*  (returns)	: KBSQLSelect * : Select query class or NULL on error	*/

KBSQLSelect
	*KBTestDriver::qrySelect
	(	bool		data,
		const QString	&select,
		bool
	)
{
	return	new KBTestDriverQrySelect (this, data, select) ;
}


/*  KBTestDriver							*/
/*  qryUpdate	: Open an update query					*/
/*  data	: bool		  : Query for data			*/
/*  update	: const QString & : Update query			*/
/*  tabName	: const QString & : Table being updated			*/
/*  (returns)	: KNQryUpdate *   : Update query class or NULL on error	*/

KBSQLUpdate
	*KBTestDriver::qryUpdate
	(	bool		data,
		const QString	&update,
		const QString	&tabName
	)
{
	return	new KBTestDriverQryUpdate (this, data, update, tabName) ;
}

/*  KBTestDriver							*/
/*  qryInsert	: Open an insert query					*/
/*  data	: bool		  : Query for data			*/
/*  insert	: const QString & : Insert query			*/
/*  tabName	: const QString & : Table being updated			*/
/*  (returns)	: KNQryInsert *   : Insert query class or NULL on error	*/

KBSQLInsert
	*KBTestDriver::qryInsert
	(	bool		data,
		const QString	&insert,
		const QString	&tabName
	)
{
	return	new KBTestDriverQryInsert (this, data, insert, tabName) ;
}

/*  KBTestDriver							*/
/*  qryDelete	: Open an delete query					*/
/*  data	: bool		  : Query for data			*/
/*  _delete	: const QString & : Delete query			*/
/*  tabName	: const QString & : Table being updated			*/
/*  (returns)	: KNQryDelete *   : Delete query class or NULL on error	*/

KBSQLDelete
	*KBTestDriver::qryDelete
	(	bool		data,
		const QString	&_delete,
		const QString	&tabName
	)
{
	return	new KBTestDriverQryDelete (this, data, _delete, tabName) ;
}

/*  KBTestDriver							*/
/*  command	: Execute arbitrary SQL					*/
/*  data	: bool		  : Querying for data			*/
/*  rawqry	: const QString & : Query text				*/
/*  nvals	: uint		  : Number of substitution values	*/
/*  values	: KBValue *	  : Substitution values			*/
/*  select	: KBSQLSelect **  : Return for result queries		*/
/*  (returns)	: bool		  : Success				*/

bool	KBTestDriver::command
	(	bool		,
		const QString	&,
		uint		,
		KBValue		*,
		KBSQLSelect	**
	)
{
	m_lError = KBError
		   (	KBError::Error,
			"Not implemented",
			"create",
			__ERRLOCN
		   )	;

	return	false	;
}

/*  KBTestDriver							*/
/*  listTypes	: Get list of types with information flags		*/
/*  (returns)	: QString	: List as bar-separated string		*/

QString	KBTestDriver::listTypes ()
{
	static	QString	typeList ;

	if (typeList.isNull())
	{
		typeList = "Primary Key,0|Foreign Key,0" ;

		for (uint idx = 0 ; idx < sizeof(typeMap)/sizeof(TestDriverTypeMap) ; idx += 1)
		{
			TestDriverTypeMap *m = &typeMap[idx] ;

			if ((m->flags & FF_NOCREATE) == 0)
				typeList += QString("|%1,%2").arg(m->mtype).arg(m->flags) ;
		}
	}

	return	typeList ;
}

/*  KBTestDriver							*/
/*  tableExists	: See if named table exists				*/
/*  table	: const QString & : Table name				*/
/*  exists	: bool &	  : True if table exists		*/
/*  (returns)	: bool		  : Success				*/

bool	KBTestDriver::tableExists
	(	const QString	&,
		bool		&exists
	)
{
	exists = false	; 
	return	 true	;
}

/*  KBTestDriver							*/
/*  doListTables: List tables in database				*/
/*  tabList	: KBTableDetailsList &					*/
/*				: Result list				*/
/*  type	: uint		: Type flags				*/
/*  (returns)	: bool		: Success				*/

bool	KBTestDriver::doListTables
	(	KBTableDetailsList	&tabList,
		uint
	)
{
	const char *location	= getenv("TESTPATH") ;
        QString	   path		;

	if (location == 0)
		path	= locateDir("appdata", "test/dummy/TableA.xml") ;
	else	path	= location ;

//	fprintf
//	(	stderr,
//		"KBTestDriver::doListTables: l=[%s] p=[%s]\n",
//		location,
//		path.latin1()
//	)	;

	QDir dir (path + "test/dummy", "*.xml") ;
	for  (uint idx = 0 ; idx < dir.count() ; idx += 1)
	{
		QString	name = dir[idx] ;
		name = name.left(name.length() - 4) ;

//		fprintf
//		(	stderr,
//			"KBTestDriver::doListTables: t=[%s]\n",
//			name.latin1()
//		)	;
		tabList.append (KBTableDetails (name, KB::IsTable, 0xff)) ;
	}

	return	true ;
}

/*  KBTestDriver							*/
/*  doListFields: List fields in table					*/
/*  tabSpec	: KBTableSpec &	: Table specification			*/
/*  (returns)	: bool		: Success				*/

bool	KBTestDriver::doListFields
	(	KBTableSpec	&tabSpec
	)
{
	KBDomDocument	doc	;
	if (!doc.loadFile (tabSpec.m_name + ".xml", "test/dummy", getenv("TESTPATH")))
	{
		m_lError = doc.lastError() ;
		return	false	;
	}

	uint	colno	= 0	;

	DOMITER_BEGIN(doc.documentElement(), "column", column)
	{
		QString	flagStr	= column.attribute("flags") ;
		uint	itype	= column.attribute("itype"    ).toInt() ;
		int	length	= column.attribute("length"   ).toInt() ;
		int	prec	= column.attribute("precision").toInt() ;

		uint	flags	= 0 ;
		for (uint idx = 0 ; idx < flagStr.length() ; idx += 1)
			switch (flagStr.at(idx).latin1())
			{
				case 'P' : flags |= KBFieldSpec::Primary  ; break ;
				case 'N' : flags |= KBFieldSpec::NotNull  ; break ;
				case 'U' : flags |= KBFieldSpec::Unique   ; break ;
				case 'S' : flags |= KBFieldSpec::Serial   ; break ;
				case 'I' : flags |= KBFieldSpec::Indexed  ; break ;
				case 'R' : flags |= KBFieldSpec::ReadOnly ; break ;
				case 'A' : flags |= KBFieldSpec::InsAvail ; break ;
				default	 : 				    break ;
			}

		KBFieldSpec *fSpec = new KBFieldSpec
				     (		colno,
						column.attribute("name"),
						column.attribute("mtype"),
						(KB::IType)itype,
						flags,
						length,
						prec
				     ) ;

		fSpec->m_dbType	   = new KBType
				     (		"Test",
						(KB::IType)itype,
						length,
						prec,
						(((flags & KBFieldSpec::NotNull) == 0) || ((flags & KBFieldSpec::Serial) != 0))
				     ) ;

		tabSpec.m_fldList.append (fSpec) ;

		if (((flags & KBFieldSpec::Primary) != 0) && ((flags & KBFieldSpec::Serial) != 0))
			tabSpec.m_prefKey = colno ;
			
		colno	+= 1 ;
	}
	DOMITER_END

	return	true ;
}

/*  KBTestDriver							*/
/*  doCreateTable: Create a new table					*/
/*  tabSpec	 : KBTableSpec &: Table specification			*/
/*  assoc	 : bool		: Create associated objects		*/
/*  best	 : bool		: Use best match			*/
/*  (returns)	 : bool		: Success				*/

bool	KBTestDriver::doCreateTable
	(	KBTableSpec	&,
		bool		,
		bool
	)
{
	m_lError = KBError
		   (	KBError::Error,
			"Not implemented",
			"create",
			__ERRLOCN
		   )	;
	return	false	;
}

/*  KBTestDriver							*/
/*  doRenameTable: Rename a  table					*/
/*  oldName	 : cchar *	: Current table name			*/
/*  newName	 : cchar *	: New table name			*/
/*  assoc	 : bool		: Rename associated stuff - none here	*/
/*  (returns)	 : bool		: Success				*/

bool	KBTestDriver::doRenameTable
	(	cchar	*,
		cchar	*,
		bool
	)
{
	m_lError = KBError
		   (	KBError::Error,
			"Not implemented",
			"rename",
			__ERRLOCN
		   )	;
	return	false	;
}

/*  KBTestDriver							*/
/*  doDropTable	: Drop a table						*/
/*  table	: cchar *	: Table name				*/
/*  assoc	: bool		: Drop associated stuff - none here	*/
/*  (returns)	: bool		: Success				*/

bool	KBTestDriver::doDropTable
	(	cchar	*,
		bool
	)
{
	m_lError = KBError
		   (	KBError::Error,
			"Not implemented",
			"drop",
			__ERRLOCN
		   )	;
	return	false	;
}

/*  KBTestDriver							*/
/*  mapExpression: Map expressions with double-quotes			*/
/*  expr	 : const QString & : Expression				*/
/*  (returns)	 : QString	   : Mapped expression			*/

QString	KBTestDriver::mapExpression
	(	const QString	&expr
	)
{
	static	QString	spec ("_") ;

	return	m_mapExpressions ?
			doMapExpression (expr, "\"", "\"", spec) :
			expr ;
}


/*  ------------------------------------------------------------------- */

/*  KBTestDriverQrySelect						*/
/*  KBTestDriverQrySelect						*/
/*		: Constructor for select query object			*/
/*  server	: KBTestDriver *  : Server connection object		*/
/*  data	: bool		  : Query for data			*/
/*  query	: const QString & : Select query			*/
/*  (returns)	: KBTestDriverQrySelect :				*/

KBTestDriverQrySelect::KBTestDriverQrySelect
	(	KBTestDriver		*server,
		bool			data,
		const QString		&query
	)	
	:
	KBSQLSelect	(server, data, query),
	m_server	(server)
{
	m_query		= query	;
	m_nRows		= 0	;
	m_nFields	= 0	;
	m_crow	 	= 0	;
	m_lengths	= 0	;
}

/*  KBTestDriverQrySelect							*/
/*  ~KBTestDriverQrySelect							*/
/*		: Destructor for select query object			*/
/*  (returns)	:		:					*/

KBTestDriverQrySelect::~KBTestDriverQrySelect ()
{
}

/*  KBTestDriverQrySelect							*/
/*  execute	: Execute query						*/
/*  nvals	: uint		: Number of substitution values		*/
/*  values	: KBValue *	: Substitution values			*/
/*  (returns)	: bool		: Success				*/

bool	KBTestDriverQrySelect::execute
	(	uint		,
		const KBValue	*
	)
{
	QRegExp		split	("[ ,]+") ;
	QStringList	bits	= QStringList::split (split, m_query)	;
	uint		idx	= 1 	;
	bool		random	= false	;

	if (bits[idx] == "distinct")
	{	random	= true	;
		idx    += 1	;
	}

	while (idx < bits.count())
	{
		if (bits[idx] == "from")
		{	m_table	= bits[idx+1] ;
			idx    += 2 ;
			break	;
		}

		QStringList parts = QStringList::split ('.', bits[idx]) ;
//		m_columns.append (bits[idx]) ;
		m_columns.append (parts[parts.count() - 1]) ;
		idx += 1 ;
	}

	KBDomDocument	doc	;
	if (!doc.loadFile (m_table + ".xml", "test/dummy", getenv("TESTPATH")))
	{
		m_lError = doc.lastError() ;
		return	false	;
	}

	QDomNodeList nl	= doc.documentElement().elementsByTagName("record") ;

	if (random)
	{
		static	bool	first	= true ;

		if (first)
		{
			QTime t	= QTime::currentTime() ;
			srand (t.hour() * 12 + t.minute() * 60 + t.second() * 60) ;
			first	= false ;
		}
	}

	for (uint idx = 0 ; idx < nl.count() ; idx += 1)
	{
		if (!nl.item(idx).isElement())
			continue ;

		if (!random || (rand() % 2 == 0))
			m_rows.append (nl.item(idx).toElement()) ;
	}

	m_nRows		= m_rows   .count() ;
	m_nFields	= m_columns.count() ;


	if (m_types == 0)
	{
		m_types	= new KBType *[m_nFields] ;
		for (uint idx = 0 ; idx < m_nFields ; idx += 1)
		{
			m_types[idx] = 0 ;

			DOMITER_BEGIN(doc.documentElement(),"column",column)
			{
				if (m_columns[idx] == column.attribute("name"))
				{
					m_types[idx] = new KBType
							   (	"Test",
								(KB::IType)column.attribute("itype").toInt(),
								column.attribute("length"   ).toInt(),
								column.attribute("precision").toInt(),
								true
							   )	;
					break	;
				}
			}
			DOMITER_END

			if (m_types[idx] == 0)
				m_types[idx] = new KBType
						   (	"Test",
						   	KB::ITString,
						   	255,
						   	0,
						   	true
						   )	;
		}
	}

	return	true	;
}

/*  KBTestDriverQrySelect							*/
/*  getField	: Get a specified field from the query results		*/
/*  qrow	: uint		: Row number				*/
/*  qcol	: uint		: Column number				*/
/*  (returns)	: KBValue	: Value					*/

KBValue	KBTestDriverQrySelect::getField
	(	uint		qrow,
		uint		qcol,
		KBValue::VTrans
	)
{
	/* First check that the request is within the range of rows	*/
	/* and fields returned by the query.				*/
	if ((int)qrow >= m_nRows  ) return KBValue() ;
	if (     qcol >= m_nFields) return KBValue() ;

	QDomElement record = m_rows[qrow] ; //.toElement() ;
	QString	    value  = record.attribute(m_columns[qcol]) ;

	return	value.isNull() ? KBValue() : KBValue(value, m_types[qcol]) ;
}

/*  KBTestDriverQrySelect							*/
/*  getFieldname: Get a specified field name from the query results	*/
/*  qcol	: uint		: Column number				*/
/*  (returns)	: QString	: Name					*/

QString	KBTestDriverQrySelect::getFieldName
	(	uint	qcol
	)
{
	/* First check that the request is within the range of rows	*/
	/* and fields returned by the query.				*/
	if (qcol >= m_nFields) return QString() ;

	return	QString::null ;
}


/*  KBTestDriverQryUpdate							*/
/*  KBTestDriverQryUpdate							*/
/*		: Constructor for update query object			*/
/*  server	: KBTestDriver *	  : Server connection object		*/
/*  data	: bool		  : Query for data			*/
/*  query	: const QString & : Update query			*/
/*  tabName	: const QString & : Table being updated			*/
/*  (returns)	: KBTestDriverQryUpdate:					*/

KBTestDriverQryUpdate::KBTestDriverQryUpdate
	(	KBTestDriver		*server,
		bool		data,
		const QString	&query,
		const QString	&tabName
	)	
	:
	KBSQLUpdate (server, data, query, tabName),
	m_server    (server)
{
	m_nRows	= 0 ;
}

/*  KBTestDriverQryUpdate							*/
/*  execute	: Execute query						*/
/*  nvals	: uint		: Number of substitution values		*/
/*  values	: KBValue *	: Substitution values			*/
/*  (returns)	: bool		: Success				*/

bool	KBTestDriverQryUpdate::execute
	(	uint		,
		const KBValue	*
	)
{
	m_lError = KBError
		   (	KBError::Error,
			"Not implemented",
			"update",
			__ERRLOCN
		   )	;
	return	false	;
}

/*  KBTestDriverQryUpdate							*/
/*  ~KBTestDriverQryUpdate							*/
/*		: Destructor for update query object			*/
/*  (returns)	:		:					*/

KBTestDriverQryUpdate::~KBTestDriverQryUpdate ()
{
}


/*  KBTestDriverQryInsert							*/
/*  KBTestDriverQryInsert							*/
/*		: Constructor for insert query object			*/
/*  server	: KBTestDriver *	  : Server connection object		*/
/*  data	: bool		  : Query for data			*/
/*  tabName	: const QString & : Table being updated			*/
/*  query	: const QString & : Insert query			*/
/*  (returns)	: KBTestDriverQryInsert:					*/

KBTestDriverQryInsert::KBTestDriverQryInsert
	(	KBTestDriver		*server,
		bool		data,
		const QString	&query,
		const QString	&tabName
	)	
	:
	KBSQLInsert (server, data, query, tabName),
	m_server    (server)
{
	m_nRows	= 0 ;
}

/*  KBTestDriverQryInsert							*/
/*  ~KBTestDriverQryInsert							*/
/*		: Destructor for insert query object			*/
/*  (returns)	:		:					*/

KBTestDriverQryInsert::~KBTestDriverQryInsert ()
{
}

/*  KBTestDriverQryInsert							*/
/*  execute	: Execute query						*/
/*  nvals	: uint		: Number of substitution values		*/
/*  values	: KBValue *	: Substitution values			*/
/*  (returns)	: bool		: Success				*/

bool	KBTestDriverQryInsert::execute
	(	uint		,
		const KBValue	*
	)
{
	m_lError = KBError
		   (	KBError::Error,
			"Not implemented",
			"insert",
			__ERRLOCN
		   )	;
	return	false	;
}

/*  KBTestDriverQryInsert							*/
/*  getNewKey	: Get new insert key					*/
/*  primary	: const QString & : Key column name			*/
/*  _newKey	: KBValue &	  : New key				*/
/*  prior	: bool		  : Pre-insert call			*/
/*  (returns)	: bool		  : Success				*/

bool	KBTestDriverQryInsert::getNewKey
	(	const QString	&,
		KBValue		&,
		bool		
	)
{
	return	false	;
}

/*  KBTestDriverQryDelete							*/
/*  KBTestDriverQryDelete							*/
/*		: Constructor for delete query object			*/
/*  server	: KBTestDriver *	  : Server connection object		*/
/*  data	: bool		  : Query for data			*/
/*  query	: const QString & : Delete query			*/
/*  tabName	: const QString & : Table being updated			*/
/*  (returns)	: KBTestDriverQryDelete:					*/

KBTestDriverQryDelete::KBTestDriverQryDelete
	(	KBTestDriver		*server,
		bool		data,
		const QString	&query,
		const QString	&tabName
	)	
	:
	KBSQLDelete (server, data, query, tabName),
	m_server    (server)
{
	m_nRows	= 0 ;
}

/*  KBTestDriverQryDelete							*/
/*  ~KBTestDriverQryDelete							*/
/*		: Destructor for delete query object			*/
/*  (returns)	:		:					*/

KBTestDriverQryDelete::~KBTestDriverQryDelete ()
{
}

/*  KBTestDriverQryDelete							*/
/*  execute	: Execute query						*/
/*  nvals	: uint		: Number of substitution values		*/
/*  values	: KBValue *	: Substitution values			*/
/*  (returns)	: bool		: Success				*/

bool	KBTestDriverQryDelete::execute
	(	uint		,
		const KBValue	*
	)
{
	m_lError = KBError
		   (	KBError::Error,
			"Not implemented",
			"delete",
			__ERRLOCN
		   )	;
	return	false	;
}


KBFACTORY
(	KBTestDriverFactory,
	"driver_dummy"
)

KBFACTORYIMPL
(	KBTestDriverFactory,
	driver_dummy
)

QObject	*KBTestDriverFactory::create
	(	QObject		  *parent,
		cchar		  *object,
		cchar		  *,
		const QStringList &
	)
{
	if (dIdentToType.count() == 0)
		for (uint idx = 0 ; idx < sizeof(typeMap)/sizeof(TestDriverTypeMap) ; idx += 1)
			if (typeMap[idx].ident != FIELD_TYPE_UNSET)
			{
				TestDriverTypeMap *m = &typeMap[idx] ;
				dIdentToType.insert (m->ident, m) ;
			}

	if ((parent != 0) && !parent->inherits ("QWidget"))
	{
		fprintf	(stderr, "KBTestDriverFactory: parent does not inherit QWidget\n") ;
		return	0  ;
	}

	if (strcmp (object, "driver"  ) == 0) return new KBTestDriver () ;

	if (strcmp (object, "advanced") == 0) return 0 ;

	return	0 ;
}

QString	KBTestDriverFactory::ident ()
{
	return	"Dummty test driver" ;
}


EXPORT_API
	KBServer*createTestDriver
	(	const QMap<QString,QString>	&options
	)
{
	return	new KBTestDriver (options) ;
}
